@@ -1,7 +1,6 @@
-integer, external :: grib_f_open_file, grib_f_close_file, grib_f_read_file
+integer, external :: grib_f_open_file, grib_f_close_file, &
+                     grib_f_read_file,grib_f_write_file
 integer, external :: grib_f_multi_support_on, grib_f_multi_support_off
-integer, external :: grib_f_context_set_real, &
-                     grib_f_context_get_real
 integer, external :: grib_f_keys_iterator_new, &
                      grib_f_keys_iterator_next, &
                      grib_f_keys_iterator_delete
@@ -17,6 +16,7 @@ integer, external :: grib_f_new_from_message, &
                      grib_f_new_from_message_copy, &
                      grib_f_new_from_template, &
                      grib_f_new_from_samples, &
+                     grib_f_read_any_from_file, &
                      grib_f_new_from_file
 integer, external :: grib_f_release
 integer, external :: grib_f_dump, grib_f_print
@@ -32,13 +32,17 @@ integer, external :: grib_f_get_int, grib_f_get_long,grib_f_get_int_array, &
 					 grib_f_get_string,grib_f_is_missing
 integer, external :: grib_f_new_from_index, &
                      grib_f_index_new_from_file, &
+                     grib_f_index_read, &
+                     grib_f_index_write, &
                      grib_f_index_release, &
                      grib_f_index_get_size_long, &
                      grib_f_index_get_size_int, &
                      grib_f_index_get_int, &
                      grib_f_index_get_long, &
+					 grib_f_index_get_string, &
                      grib_f_index_get_real8, &
                      grib_f_index_select_real8, &
+                     grib_f_index_select_string, &
                      grib_f_index_select_int, &
                      grib_f_index_select_long
                          
