@@ -1,5 +1,5 @@
 public :: grib_get, grib_set, grib_get_data, grib_is_missing
-public :: grib_open_file, grib_close_file
+public :: grib_open_file, grib_close_file,grib_read_bytes,grib_write_bytes
 public :: grib_multi_support_on, grib_multi_support_off
 public :: grib_keys_iterator_new, &
           grib_keys_iterator_next, &
@@ -10,10 +10,9 @@ public :: grib_skip_computed, &
           grib_skip_read_only
 public :: grib_keys_iterator_get_name, &
           grib_keys_iterator_rewind
-public :: grib_new_from_message, &
-          grib_new_from_template, &
-          grib_new_from_samples, &
-          grib_new_from_file
+public :: grib_new_from_message, grib_new_from_template, &
+          grib_new_from_samples, grib_new_from_file, &
+		  grib_read_from_file
 public :: grib_release
 public :: grib_dump
 public :: grib_get_error_string
