@@ -571,7 +571,7 @@ static int clear_file(int file_id){
 static int _clear_handle(int handle_id){
 
   l_grib_handle* current  = handle_set;
-
+  if (handle_id<0) return 0;
   while(current){
     if(current->id == handle_id){
       current->id = -(current->id);
@@ -683,6 +683,52 @@ static int clear_keys_iterator(int keys_iterator_id){
   return ret;
 }
 
+int grib_f_read_any_from_file_(int* fid, char* buffer, int* nbytes) {
+  grib_context* c;
+  int err=0;
+  size_t size=(size_t)*nbytes;
+  FILE* f=get_file(*fid);
+
+  if (f) {
+    c=grib_context_get_default( );
+    err=grib_read_any_from_file(c,f,buffer,&size);
+	*nbytes=size;
+	return err;
+  } else {
+    return GRIB_INVALID_FILE;
+  }
+}
+int grib_f_read_any_from_file__(int* fid, char* buffer, int* nbytes) {
+	return grib_f_read_any_from_file_(fid,buffer,nbytes);
+}
+int grib_f_read_any_from_file(int* fid, char* buffer, int* nbytes) {
+	return grib_f_read_any_from_file_(fid,buffer,nbytes);
+}
+
+int grib_f_write_file_(int* fid, char* buffer, int* nbytes) {
+  grib_context* c;
+  FILE* f=get_file(*fid);
+
+  if (f) {
+    int ioerr;
+    c=grib_context_get_default( );
+    if( fwrite(buffer, 1, *nbytes, f)  != *nbytes) {
+       ioerr=errno;
+       grib_context_log(c,(GRIB_LOG_ERROR)|(GRIB_LOG_PERROR),"IO ERROR: %s",strerror(ioerr));
+       return GRIB_IO_PROBLEM;
+    }
+    return GRIB_SUCCESS;
+  } else {
+    return GRIB_INVALID_FILE;
+  }
+}
+int grib_f_write_file__(int* fid, char* buffer, int* nbytes) {
+  return grib_f_write_file_(fid,buffer,nbytes);
+}
+int grib_f_write_file(int* fid, char* buffer, int* nbytes) {
+  return grib_f_write_file_(fid,buffer,nbytes);
+}
+
 int grib_f_read_file_(int* fid, char* buffer, int* nbytes) {
   grib_context* c;
   FILE* f=get_file(*fid);
@@ -1256,6 +1302,52 @@ int grib_f_index_new_from_file(char* file ,                                char*
   return grib_f_index_new_from_file_(file ,keys ,gid, lfile, lkeys);
 }
 
+int grib_f_index_read_(char* file, int* gid, int lfile) {
+  int err = 0;
+  char fname[1024]={0,};
+
+  grib_index *i = NULL;
+
+  if (*file) {
+    i = grib_index_read(0,cast_char(fname,file,lfile),&err);
+    if (i) {
+      push_index(i,gid);
+      return GRIB_SUCCESS;
+    } else {
+      *gid = -1;
+      return GRIB_END_OF_FILE;
+    }
+  }
+
+  *gid=-1;
+  return GRIB_INVALID_FILE;
+}
+int grib_f_index_read__(char* file, int* gid, int lfile) {
+  return grib_f_index_read_(file,gid,lfile);
+}
+int grib_f_index_read(char* file, int* gid, int lfile) {
+  return grib_f_index_read_(file,gid,lfile);
+}
+
+int grib_f_index_write_(int* gid, char* file, int lfile) {
+  grib_index *i = get_index(*gid);
+  int err = GRIB_SUCCESS;
+  char buf[1024];
+
+  if (!i) {
+    return GRIB_INVALID_GRIB;
+  } else {
+    err = grib_index_write(i,cast_char(buf,file,lfile));
+    return err;
+  }
+}
+int grib_f_index_write__(int* gid, char* file, int lfile) {
+  return grib_f_index_write_(gid,file,lfile);
+}
+int grib_f_index_write(int* gid, char* file, int lfile) {
+  return grib_f_index_write_(gid,file,lfile);
+}
+
 int grib_f_index_release_(int* hid){
   return clear_index(*hid);
 }
@@ -1519,6 +1611,48 @@ int grib_f_get_long_array(int* gid, char* key, long *val, int* size, int len){
   return grib_f_get_long_array_( gid,  key, val,  size,  len);
 }
 
+int grib_f_index_get_string_(int* gid, char* key, char* val, int *eachsize,int* size,  int len){
+
+  grib_index *h = get_index(*gid);
+  int err = GRIB_SUCCESS;
+  int i;
+  char buf[1024];
+  size_t lsize = *size;
+  char** bufval;
+  char* p=val;
+
+  if(!h)  return GRIB_INVALID_GRIB;
+
+  bufval=(char**)grib_context_malloc_clear(h->context,sizeof(char*)* *size);
+
+  err = grib_index_get_string(h, cast_char(buf,key,len), bufval, &lsize);
+    *size = lsize;
+
+  if (err) return err;
+
+  for (i=0;i<lsize;i++) {
+    int l=strlen(bufval[i]);
+	int j;
+    if (*eachsize < l ) {
+	  grib_context_free(h->context,bufval);
+	  printf("eachsize=%d strlen(bufval[i])=%ld\n",*eachsize,(long)strlen(bufval[i]));
+	  return GRIB_ARRAY_TOO_SMALL;
+	}
+	memcpy(p,bufval[i],l);
+	p+=l;
+	for (j=0;j<*eachsize-l;j++) *(p++)=' ';
+  }
+  grib_context_free(h->context,bufval);
+
+  return  err;
+}
+int grib_f_index_get_string__(int* gid, char* key, char *val, int* eachsize, int* size,  int len){
+  return grib_f_index_get_string_(gid,key,val,eachsize,size,len);
+}
+int grib_f_index_get_string(int* gid, char* key, char* val, int* eachsize, int* size,  int len){
+  return grib_f_index_get_string_(gid,key,val,eachsize,size,len);
+}
+
 int grib_f_index_get_long_(int* gid, char* key, long *val, int* size,  int len){
 
   grib_index *h = get_index(*gid);
@@ -1871,6 +2005,23 @@ int grib_f_index_select_real8(int* gid, char* key, double* val, int len){
   return grib_f_index_select_real8_(gid,key,val,len);
 }
 
+int grib_f_index_select_string_(int* gid, char* key, char* val, int len, int vallen){
+
+  grib_index *h = get_index(*gid);
+
+  char buf[1024];
+  char bufval[1024];
+
+  if(!h) return GRIB_INVALID_GRIB;
+  return grib_index_select_string(h, cast_char(buf,key,len), cast_char(bufval,val,vallen));
+}
+int grib_f_index_select_string__(int* gid, char* key, char* val, int len, int vallen){
+  return grib_f_index_select_string_(gid,key,val,len,vallen);
+}
+int grib_f_index_select_string(int* gid, char* key, char* val, int len, int vallen){
+  return grib_f_index_select_string_(gid,key,val,len,vallen);
+}
+
 int grib_f_index_select_int_(int* gid, char* key, int* val, int len){
 
   grib_index *h = get_index(*gid);
