@@ -1,5 +1,11 @@
 
 /* grib_fortran.c */
+int grib_f_read_any_from_file_(int *fid, char *buffer, int *nbytes);
+int grib_f_read_any_from_file__(int *fid, char *buffer, int *nbytes);
+int grib_f_read_any_from_file(int *fid, char *buffer, int *nbytes);
+int grib_f_write_file_(int *fid, char *buffer, int *nbytes);
+int grib_f_write_file__(int *fid, char *buffer, int *nbytes);
+int grib_f_write_file(int *fid, char *buffer, int *nbytes);
 int grib_f_read_file_(int *fid, char *buffer, int *nbytes);
 int grib_f_read_file__(int *fid, char *buffer, int *nbytes);
 int grib_f_read_file(int *fid, char *buffer, int *nbytes);
@@ -93,6 +99,12 @@ int grib_f_new_from_index(int *iid, int *gid);
 int grib_f_index_new_from_file_(char *file, char *keys, int *gid, int lfile, int lkeys);
 int grib_f_index_new_from_file__(char *file, char *keys, int *gid, int lfile, int lkeys);
 int grib_f_index_new_from_file(char *file, char *keys, int *gid, int lfile, int lkeys);
+int grib_f_index_read_(char* file, int* gid, int lfile);
+int grib_f_index_read__(char* file, int* gid, int lfile);
+int grib_f_index_read(char* file, int* gid, int lfile);
+int grib_f_index_write_(int* gid, char* file, int lfile);
+int grib_f_index_write__(int* gid, char* file, int lfile);
+int grib_f_index_write(int* gid, char* file, int lfile);
 int grib_f_index_release_(int *hid);
 int grib_f_index_release__(int *hid);
 int grib_f_index_release(int *hid);
@@ -135,6 +147,9 @@ int grib_f_get_int_array(int *gid, char *key, int *val, int *size, int len);
 int grib_f_get_long_array_(int *gid, char *key, long *val, int *size, int len);
 int grib_f_get_long_array__(int *gid, char *key, long *val, int *size, int len);
 int grib_f_get_long_array(int *gid, char *key, long *val, int *size, int len);
+int grib_f_index_get_string_(int *gid, char *key, char *val, int *eachsize, int *size, int len);
+int grib_f_index_get_string__(int *gid, char *key, char *val, int *eachsize, int *size, int len);
+int grib_f_index_get_string(int *gid, char *key, char *val, int *eachsize, int *size, int len);
 int grib_f_index_get_long_(int *gid, char *key, long *val, int *size, int len);
 int grib_f_index_get_long__(int *gid, char *key, long *val, int *size, int len);
 int grib_f_index_get_long(int *gid, char *key, long *val, int *size, int len);
@@ -183,6 +198,9 @@ int grib_f_set_real4_array(int *gid, char *key, float *val, int *size, int len);
 int grib_f_index_select_real8_(int *gid, char *key, double *val, int len);
 int grib_f_index_select_real8__(int *gid, char *key, double *val, int len);
 int grib_f_index_select_real8(int *gid, char *key, double *val, int len);
+int grib_f_index_select_string_(int *gid, char *key, char *val, int len, int vallen);
+int grib_f_index_select_string__(int *gid, char *key, char *val, int len, int vallen);
+int grib_f_index_select_string(int *gid, char *key, char *val, int len, int vallen);
 int grib_f_index_select_int_(int *gid, char *key, int *val, int len);
 int grib_f_index_select_int__(int *gid, char *key, int *val, int len);
 int grib_f_index_select_int(int *gid, char *key, int *val, int len);
