@@ -20,6 +20,7 @@
    IMPLEMENTS = destroy;execute
    MEMBERS    = grib_expression *expression
    MEMBERS    = char *name
+   MEMBERS    = int nofail
    END_CLASS_DEF
 
  */
@@ -46,6 +47,7 @@ typedef struct grib_action_set {
 /* Members defined in set */
 	grib_expression *expression;
 	char *name;
+	int nofail;
 } grib_action_set;
 
 
@@ -76,8 +78,7 @@ static void init_class(grib_action_class* c)
 /* END_CLASS_IMP */
 
 grib_action* grib_action_create_set( grib_context* context,
-  const char* name,
-    grib_expression* expression)
+  const char* name,    grib_expression* expression,int nofail)
 {
   char buf[1024];
 
@@ -92,6 +93,7 @@ grib_action* grib_action_create_set( grib_context* context,
 
   a->expression  = expression;
   a->name        = grib_context_strdup_persistent(context,name);
+  a->nofail      = nofail;
 
 
   sprintf(buf,"set%p",(void*)expression);
@@ -106,6 +108,7 @@ static int execute(grib_action* a, grib_handle *h)
   int ret=0;
   grib_action_set* self = (grib_action_set*) a;
   ret=grib_set_expression(h,self->name,self->expression);
+  if (self->nofail) return 0;
   if (ret != GRIB_SUCCESS) {
     grib_context_log(h->context,GRIB_LOG_ERROR,"Error while setting key %s (%s)",
                      self->name,grib_get_error_message(ret));
