@@ -112,6 +112,21 @@ static void dump( grib_action* act, FILE* f, int lvl)
   grib_context_print(act->context,f,"Template %s  %s\n",act->name , a->arg );
 }
 
+static char fpath[1024]={0,};
+static grib_action* get_empty_template(grib_context* c,int *err) {
+	char fname[]="empty_template.def";
+	
+	if (*fpath !=0 || grib_context_full_path(c,fname,fpath)) {
+		*err=GRIB_SUCCESS;
+		return grib_parse_file(c, fpath);
+	} else {
+		*err=GRIB_INTERNAL_ERROR;
+		grib_context_log(c,GRIB_LOG_FATAL,"unable to find %s\n",fpath);
+		return NULL;
+	}
+	
+}
+
 static int  create_accessor(grib_section* p, grib_action* act, grib_loader *h ){
   int ret = GRIB_SUCCESS;
   grib_action_template* a = ( grib_action_template*)act;
@@ -127,19 +142,23 @@ static int  create_accessor(grib_section* p, grib_action* act, grib_loader *h ){
   if(!as) return GRIB_INTERNAL_ERROR;
   if(a->arg){
     char fpath[1024]={0,};
+	
     ret = grib_recompose_name(p->h,as,a->arg,fname,1);
 
-    if (!grib_context_full_path(p->h->context,fname,fpath)) {
+	if (!grib_context_full_path(p->h->context,fname,fpath)) {
       if (!a->nofail) {
         grib_context_log(p->h->context,GRIB_LOG_ERROR,
                          "Unable to find template %s from %s ",act->name,fname);
         return GRIB_FILE_NOT_FOUND;
       }
+	  la = get_empty_template(p->h->context,&ret);
+	  if (ret) return ret;
     } else 
-        la = grib_parse_file(p->h->context, fpath);
+      la = grib_parse_file(p->h->context, fpath);
   }
   as->flags |= GRIB_ACCESSOR_FLAG_HIDDEN;
-  gs = grib_get_sub_section(as);
+  /*gs = grib_get_sub_section(as);*/
+  gs = as->sub_section;
   gs->branch = la; /* Will be used to prevent unecessary reparse */
 
   grib_push_accessor(as,p->block);
