@@ -256,6 +256,7 @@ long grib_accessor_get_native_type(grib_accessor* a)
   return 0;
 }
 
+#if 0
 grib_section* grib_get_sub_section(grib_accessor* a)
 {
   grib_accessor_class *c = NULL;
@@ -270,6 +271,12 @@ grib_section* grib_get_sub_section(grib_accessor* a)
   Assert(0);
   return 0;
 }
+#endif
+
+grib_section* grib_get_sub_section(grib_accessor* a)
+{
+	return a->sub_section;
+}
 
 long grib_get_next_position_offset(grib_accessor* a)
 {
@@ -370,6 +377,7 @@ void grib_init_accessor(grib_accessor* a, const long len, grib_arguments* args)
   init_accessor(a->cclass,a,len,args);
 }
 
+#if 0
 static void post_init_accessor(grib_accessor_class* c,grib_accessor* a)
 {
   if(c) {
@@ -378,12 +386,7 @@ static void post_init_accessor(grib_accessor_class* c,grib_accessor* a)
     if(c->post_init) c->post_init(a);
   }
 }
-
-void grib_post_init(grib_accessor* a)
-{
-  post_init_accessor(a->cclass,a);
-}
-
+#endif
 
 /* For this one, ALL destroy are called */
 
