@@ -132,6 +132,7 @@ static void init_class(grib_accessor_class* c)
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 static int grib_load_codetable(grib_context* c,const char* filename,
@@ -180,8 +181,8 @@ static void init(grib_accessor* a, const long len, grib_arguments* params) {
           len = sizeof(tmp);
           p = grib_expression_evaluate_string(a->parent->h,expression,tmp,&len,&ret);
           if (ret != GRIB_SUCCESS) {
-            grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,"unable to evaluate %s as string",a->name);
-            exit(ret);
+			  grib_context_log(a->parent->h->context,GRIB_LOG_FATAL,
+							   "unable to evaluate %s as string",a->name);
           }
           len = strlen(p)+1;
           pack_string(a,p,&len);
@@ -198,7 +199,8 @@ static grib_codetable* load_table(grib_accessor_codetable* self)
   size_t size = 0;
   grib_handle*    h = ((grib_accessor*)self)->parent->h;
   grib_context*   c = h->context;
-  grib_codetable* t = c->codetable;
+  grib_codetable* t = NULL;
+  grib_codetable* next=NULL ;
   grib_accessor* a=(grib_accessor*)self;
   char filename[1024]={0,};
   char recomposed[1024]={0,};
@@ -217,26 +219,27 @@ static grib_codetable* load_table(grib_accessor_codetable* self)
 
   if (*masterDir!=0) {
     sprintf(filename,"%s/%s",masterDir,self->tablename);
-    grib_recompose_name(h, NULL,filename, recomposed,1);
+    grib_recompose_name(h, NULL,filename, recomposed,0);
     grib_context_full_path(c,recomposed,filename);
   } else {
-    grib_recompose_name(h, NULL,self->tablename, recomposed,1);
+    grib_recompose_name(h, NULL,self->tablename, recomposed,0);
     grib_context_full_path(c,recomposed,filename);
   }
 
   if (*localDir!=0) {
     sprintf(localFilename,"%s/%s",localDir,self->tablename);
-    grib_recompose_name(h, NULL,localFilename, localRecomposed,1);
+    grib_recompose_name(h, NULL,localFilename, localRecomposed,0);
     grib_context_full_path(c,localRecomposed,localFilename);
   }
   
-  while(t) {
-    if(strcmp(filename,t->filename[0]) == 0 &&
-       ((*localFilename==0 && t->filename[1]==NULL) ||
-           ((*localFilename!=0 && t->filename[1]!=NULL)
-           && strcmp(localFilename,t->filename[1]) ==0)) )
-      return t;
-    t = t->next;
+  next=c->codetable;
+  while(next) {
+    if(strcmp(filename,next->filename[0]) == 0 &&
+       ((*localFilename==0 && next->filename[1]==NULL) ||
+           ((*localFilename!=0 && next->filename[1]!=NULL)
+           && strcmp(localFilename,next->filename[1]) ==0)) )
+      return next;
+    next = next->next;
   }
 
   if (a->flags & GRIB_ACCESSOR_FLAG_TRANSIENT) {
@@ -521,6 +524,7 @@ static int pack_string(grib_accessor* a, const char* buffer, size_t *len)
 {
   grib_accessor_codetable* self = (grib_accessor_codetable*)a;
   grib_codetable*          table ;
+
   long i;
   size_t size = 1;
 
@@ -530,11 +534,54 @@ static int pack_string(grib_accessor* a, const char* buffer, size_t *len)
   if(!table)
     return GRIB_ENCODING_ERROR;
 
+  if (a->set) {
+	  int err=grib_set_string(a->parent->h,a->set,buffer,len);
+	  if (err!=0) return err;
+  }
+
   for(i = 0 ; i < table->size; i++)
     if(table->entries[i].abbreviation)
-      if(strcmp(table->entries[i].abbreviation,buffer) == 0)
+		  if(strcmp(table->entries[i].abbreviation,buffer) == 0)
         return grib_pack_long(a,&i,&size);
 
+  if (a->flags & GRIB_ACCESSOR_FLAG_NO_FAIL) {
+	  grib_action* act=(grib_action*)(a->creator);
+	  if (act->default_value!=NULL) {
+		  const char* p = 0;
+		  size_t len = 1;
+		  long l;
+		  int ret=0;
+		  double d;
+		  char tmp[1024];
+		  grib_expression* expression=grib_arguments_get_expression(a->parent->h,act->default_value,0);
+		  int type = grib_expression_native_type(a->parent->h,expression);
+		  switch(type) {
+			  case GRIB_TYPE_DOUBLE:
+				  grib_expression_evaluate_double(a->parent->h,expression,&d);
+				  grib_pack_double(a,&d,&len);
+				  break;
+
+			  case GRIB_TYPE_LONG:
+				  grib_expression_evaluate_long(a->parent->h,expression,&l);
+				  grib_pack_long(a,&l,&len);
+				  break;
+
+			  default:
+				  len = sizeof(tmp);
+				  p = grib_expression_evaluate_string(a->parent->h,expression,tmp,&len,&ret);
+				  if (ret != GRIB_SUCCESS) {
+					  grib_context_log(a->parent->h->context,GRIB_LOG_FATAL,
+									   "unable to evaluate %s as string",a->name);
+					  return ret;
+				  }
+				  len = strlen(p)+1;
+				  pack_string(a,p,&len);
+				  break;
+		  }
+		  return GRIB_SUCCESS;
+	  }
+	
+  }
   return GRIB_ENCODING_ERROR;
 }
 
@@ -563,10 +610,10 @@ static int pack_expression(grib_accessor* a, grib_expression *e){
 
 static void destroy(grib_context* context,grib_accessor* a)
 {
-  if (a->vvalue != NULL)
-    grib_context_free_persistent(context, a->vvalue);
-
-  a->vvalue=NULL;
+	if (a->vvalue != NULL) {
+		grib_context_free(context, a->vvalue);
+		a->vvalue=NULL;
+	}
 
 }
 
