@@ -28,6 +28,7 @@
    MEMBERS=const char*  half_byte
    MEMBERS=const char*  n1
    MEMBERS=const char*  n2
+   MEMBERS=const char*  extraValues
    MEMBERS=const char*  p1
    MEMBERS=const char*  p2
    MEMBERS=const char*  matrix_values
@@ -42,6 +43,7 @@
    MEMBERS=const char*  octet_start_group
    MEMBERS=const char*  width_spd_sp_desc
    MEMBERS=const char*  nap
+   MEMBERS=const char*  bitmap
 
    END_CLASS_DEF
 
@@ -80,6 +82,7 @@ typedef struct grib_accessor_data_2order_packing {
 	const char*  half_byte;
 	const char*  n1;
 	const char*  n2;
+	const char*  extraValues;
 	const char*  p1;
 	const char*  p2;
 	const char*  matrix_values;
@@ -94,6 +97,7 @@ typedef struct grib_accessor_data_2order_packing {
 	const char*  octet_start_group;
 	const char*  width_spd_sp_desc;
 	const char*  nap;
+	const char*  bitmap;
 } grib_accessor_data_2order_packing;
 
 extern grib_accessor_class* grib_accessor_class_values;
@@ -167,6 +171,7 @@ static void init_class(grib_accessor_class* c)
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 static void init(grib_accessor* a,const long v, grib_arguments* args)
@@ -186,6 +191,7 @@ static void init(grib_accessor* a,const long v, grib_arguments* args)
   self->n1 = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->n2 = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->p1 = grib_arguments_get_name(a->parent->h,args,self->carg++);
+  self->extraValues = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->p2 = grib_arguments_get_name(a->parent->h,args,self->carg++);
 
   self->matrix_values = grib_arguments_get_name(a->parent->h,args,self->carg++);
@@ -202,35 +208,122 @@ static void init(grib_accessor* a,const long v, grib_arguments* args)
   self->octet_start_group = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->width_spd_sp_desc = grib_arguments_get_name(a->parent->h,args,self->carg++);
   self->nap = grib_arguments_get_name(a->parent->h,args,self->carg++);
+  self->bitmap = grib_arguments_get_name(a->parent->h,args,self->carg++);
   a->flags |= GRIB_ACCESSOR_FLAG_DATA;
 
 }
 
-static int      reverse_rows (unsigned long* data, long  len, long number_along_parallel){
+static const unsigned char BitsSetTable256[256] = 
+{
+#   define B2(n) n,     n+1,     n+1,     n+2
+#   define B4(n) B2(n), B2(n+1), B2(n+1), B2(n+2)
+#   define B6(n) B4(n), B4(n+1), B4(n+1), B4(n+2)
+	B6(0), B6(1), B6(1), B6(2)
+};
+
+static const unsigned char left_mask[8]={0,0x80,0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe };
+static const unsigned char right_mask[8]={0,0x80,0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe };
+
+static unsigned char* bitmap_pop_line(unsigned char* bitmap,long* bitmap_len,int *bit_offset,
+									  long points_in_line,int *values_in_line) {
+	unsigned char* p=bitmap;
+	int m=0,bits=0;
+	int bytes=0;
+
+	*values_in_line=0;
+	
+	if (*bit_offset) {
+		bits=8 - *bit_offset;
+		m=*(p++) & right_mask[bits];
+		*values_in_line += BitsSetTable256[m];
+		points_in_line-=bits;
+		(*bitmap_len)--;
+		*bit_offset=0;
+	}
+	
+	bytes=points_in_line/8;
+
+	while (bytes) {
+		m=*(p++) & 0xff;
+		*values_in_line += BitsSetTable256[m];
+		(*bitmap_len)--;
+		bytes--;
+	}
+
+	*bit_offset=points_in_line % 8;
+	m=*p & left_mask[*bit_offset];
+	*values_in_line += BitsSetTable256[m];
+	
+	return p;
+}
+
+static int reverse_rows (unsigned long* data, long  len, long number_along_parallel,
+							unsigned char* bitmap,long bitmap_len){
   long count = 0;
   long i = 0;
   long left = 0;
   long right = number_along_parallel-1;
   long tmp = 0;
+  long inc;
 
-  data += number_along_parallel;
-  count = number_along_parallel;
+  if (bitmap_len==0) {
+	/* NO BITMAP*/
+	inc=number_along_parallel;
+	count = number_along_parallel;
 
-  while(count < len){
-    left = 0;
-    right = number_along_parallel-1;
+	while(count < len){
+		left = 0;
+		right = number_along_parallel-1;
+		data += inc;
 
-    for (i = 0;i<number_along_parallel/2;i++){
-      tmp = data[left];
-      data[left] = data[right];
-      data[right] = tmp;
+		for (i = 0;i<number_along_parallel/2;i++){
+		tmp = data[left];
+		data[left] = data[right];
+		data[right] = tmp;
 
-      right--;
-      left++;
-    }
+		right--;
+		left++;
+		Assert(left<len);
+		Assert(right>0);
+		}
 
-    data  += number_along_parallel*2;
-    count += number_along_parallel*2;
+		inc=number_along_parallel*2;
+		count += inc;
+	}
+  } else {
+	  /*BITMAP present number of point per line not constant*/
+	  int line_len=0,bit_offset=0;
+	  long bitmap_left_len=bitmap_len;
+	  unsigned char* p=bitmap;
+	  long count=0;
+
+	  p=bitmap_pop_line(p,&bitmap_left_len,&bit_offset,number_along_parallel,&line_len);
+	  while(bitmap_left_len>0 && count>=len){
+		  count+=line_len;
+		  data += line_len;
+
+		  p=bitmap_pop_line(p,&bitmap_left_len,&bit_offset,number_along_parallel,&line_len);
+		  left = 0;
+		  right = line_len-1;
+		  
+		  for (i = 0;i<line_len/2;i++){
+			  tmp = data[left];
+			  data[left] = data[right];
+			  data[right] = tmp;
+
+			  right--;
+			  left++;
+			  Assert(left<len);
+			  Assert(right>0);
+		  }
+
+		  count+=line_len;
+		  Assert(count<len);
+		  
+		  data+=line_len;
+		  p=bitmap_pop_line(p,&bitmap_left_len,&bit_offset,number_along_parallel,&line_len);
+	  }
+	  
   }
 
   return 0;
@@ -326,24 +419,15 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   size_t j = 0;
   size_t n_vals = grib_value_count(a);
 
-  long     vcount = 0;
-  int      err = 0;
-  long     bias = 0;
+  long     vcount = 0, err = 0, bias = 0;
   double   reference_value;
   long     binary_scale_factor;
-  long     bits_per_value;
-  long     decimal_scale_factor;
-  long     n1 = 0;
-  long     n2 = 0;
-  long     p1 = 0;
-  long     p2 = 0;
+  long     bits_per_value, decimal_scale_factor;
+  long     n1 = 0, n2 = 0, extraValues = 0, p1 = 0, p2 = 0;
 
-  long     offsetsection = 0;
-  long     snd_bitmap = 0;
-  long     snd_ordr_wdiff = 0;
+  long     offsetsection = 0, snd_bitmap = 0, snd_ordr_wdiff = 0;
 
   long  matrix_values    =0;
-
   long  general_ext    =0;
   long  boustrophedonic =0;
   long  two_ordr_spd =0;
@@ -378,7 +462,10 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   long pointer_of_group_width = 0;
   long refsp = 0;
   long nap = 0;
-
+  unsigned char* bitmap=NULL;
+  grib_accessor* abitmap=NULL;
+  size_t bitmap_len=0;
+  
   if((err = grib_get_long_internal(a->parent->h,self->offsetsection,&offsetsection))
       != GRIB_SUCCESS) return err;
   if((err = grib_get_long_internal(a->parent->h,self->bits_per_value,&bits_per_value))
@@ -392,6 +479,10 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   if((err = grib_get_long_internal(a->parent->h,self->n1,&n1))  != GRIB_SUCCESS) return err;
   if((err = grib_get_long_internal(a->parent->h,self->n2, &n2)) != GRIB_SUCCESS) return err;
   if((err = grib_get_long_internal(a->parent->h,self->p1, &p1)) != GRIB_SUCCESS) return err;
+  if((err = grib_get_long_internal(a->parent->h,self->extraValues, &extraValues)) != GRIB_SUCCESS) return err;
+
+  p1=p1+65536*extraValues;
+  
   if((err = grib_get_long_internal(a->parent->h,self->p2, &p2)) != GRIB_SUCCESS)  return err;
   if((err = grib_get_long_internal(a->parent->h,self->matrix_values, &matrix_values))
       != GRIB_SUCCESS)return err;
@@ -413,8 +504,10 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
       != GRIB_SUCCESS) return err;
   if((err = grib_get_long_internal(a->parent->h,self->octet_start_group, &octet_start_group))
       != GRIB_SUCCESS) return err;
+
   if((err = grib_get_long_internal(a->parent->h,self->width_spd_sp_desc, &width_spd_sp_desc))
-      != GRIB_SUCCESS) return err;
+	  != GRIB_SUCCESS) width_spd_sp_desc=-1;
+
   if((err = grib_get_long_internal(a->parent->h,self->nap, &nap)) != GRIB_SUCCESS) return err;
 
   self->dirty=0;
@@ -423,6 +516,13 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
 
   Assert(bits_per_value < (sizeof(unsigned long)*8)-1);
 
+  if ((abitmap=grib_find_accessor(a->parent->h,self->bitmap))!=NULL) {
+	  bitmap_len=grib_byte_count(abitmap);
+	  bitmap=grib_context_malloc_clear(a->parent->h->context,sizeof(char)*bitmap_len);
+	  err=grib_unpack_bytes(abitmap,bitmap,&bitmap_len);
+	  if (err) {grib_context_free(a->parent->h->context,bitmap); return err;}
+  }
+
   if(bits_per_value == 0)
     return GRIB_NOT_IMPLEMENTED;
 
@@ -463,6 +563,8 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
     return GRIB_NOT_IMPLEMENTED;
 
 
+  sec_val  = grib_context_malloc(a->parent->h->context,(n_vals)*sizeof(unsigned long));
+  grib_context_free(a->parent->h->context,sec_val);
   sec_val  = grib_context_malloc(a->parent->h->context,(n_vals)*sizeof(unsigned long));
 
   buf_width_of_group  +=  a->offset;
@@ -526,8 +628,8 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
     de_spatial_difference(a->parent->h->context,sec_val, n_vals, n_sp_diff, bias);
 
   if(boustrophedonic)
-    reverse_rows      (sec_val,  n_vals,  nap);
-
+    reverse_rows(sec_val,n_vals,nap,bitmap,bitmap_len);
+  
   s = grib_power(binary_scale_factor,2);
   d = grib_power(-decimal_scale_factor,10) ;
 
@@ -547,6 +649,7 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
   max *= d;
 
   grib_context_free(a->parent->h->context,sec_val);
+  if (bitmap!=NULL) grib_context_free(a->parent->h->context,bitmap);
 
   return err;
 }
@@ -600,6 +703,9 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
   unsigned long*  group_val  = NULL;
 
   short  n_sp_diff = 0;
+  unsigned char* bitmap=NULL;
+  grib_accessor* abitmap=NULL;
+  size_t bitmap_len=0;
 
 #if KEEP_OLD == 1
 
@@ -616,6 +722,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 
   double max;
   double min;
+  int extraValues=0;
 
 
   size_t nv =0;
@@ -635,6 +742,13 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
   if((err = grib_get_long_internal(a->parent->h,self->width_spd_sp_desc, &width_spd_sp_desc)) != GRIB_SUCCESS) return err;
   if((err = grib_get_long_internal(a->parent->h,self->nap, &nap)) != GRIB_SUCCESS) return err;
 
+  if ((abitmap=grib_find_accessor(a->parent->h,self->bitmap))!=NULL) {
+	  bitmap_len=grib_byte_count(abitmap);
+	  bitmap=grib_context_malloc_clear(a->parent->h->context,sizeof(char)*bitmap_len);
+	  err=grib_unpack_bytes(abitmap,bitmap,&bitmap_len);
+	  if (err) {grib_context_free(a->parent->h->context,bitmap); return err;}
+  }
+
   two_ordr_spd = 1;
   plus1_spd = 0;
 
@@ -665,7 +779,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
   }
 
   /*  the scale factor in Grib 1 is adjusted in gribex, for "normalization purpose" ... ?*/
-  binary_scale_factor    = grib_get_binary_scale_fact(max,reference_value,bits_per_value);
+  binary_scale_factor    = grib_get_binary_scale_fact(max,reference_value,bits_per_value,&err);
 
   divisor         = grib_power(-binary_scale_factor,2);
 
@@ -675,7 +789,8 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 
   /*  reverse the rows*/
   if(boustrophedonic)
-    reverse_rows(sec_val,  n_vals,  nap);
+	reverse_rows(sec_val,n_vals,nap,bitmap,bitmap_len);
+    
 
   if(snd_ordr_wdiff)
     if((err =  spatial_difference(a->parent->h->context,sec_val, n_vals, n_sp_diff, &bias))){
@@ -772,7 +887,15 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
   n2 += 1+a->offset-offsetsection;
   if((err = grib_set_long_internal(a->parent->h,self->n2,n2)) != GRIB_SUCCESS) return err;
 
-  if((err = grib_set_long_internal(a->parent->h,self->p1,p1)) != GRIB_SUCCESS) return err;
+  	extraValues=0;
+  	while (p1 > 65535) {
+		p1-=65536;
+		extraValues++;	  
+	}
+
+	if((err = grib_set_long_internal(a->parent->h,self->p1,p1)) != GRIB_SUCCESS) return err;
+
+	if((err = grib_set_long_internal(a->parent->h,self->extraValues,extraValues)) != GRIB_SUCCESS) return err;
 
   if((err = grib_set_long_internal(a->parent->h,self->p2,n_vals - n_sp_diff)) != GRIB_SUCCESS) return err;
 
@@ -805,6 +928,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 
   grib_context_free(a->parent->h->context,buf);
   grib_context_free(a->parent->h->context,sec_val);
+  if (bitmap!=NULL) grib_context_free(a->parent->h->context,bitmap);
 
   return GRIB_SUCCESS;
 }
@@ -820,6 +944,7 @@ static long value_count(grib_accessor* a){
   long nbits_per_lengths = 0;
     long pointer_of_group_size = 0;
   long p1 = 0;
+  long extraValues=0;
 
   size_t i = 0;
 
@@ -835,6 +960,10 @@ static long value_count(grib_accessor* a){
     return 0;
   if( grib_get_long_internal(a->parent->h,self->p1, &p1) != GRIB_SUCCESS)
     return 0;
+  if( grib_get_long_internal(a->parent->h,self->extraValues, &extraValues) != GRIB_SUCCESS)
+	  return 0;
+
+  p1+=extraValues*65536;
 
   buf_size_of_groups +=  offsetsection+(octet_start_group-1);
   count = two_ordr_spd*2+plus1_spd;
