@@ -152,6 +152,7 @@ static void init_class(grib_accessor_class* c)
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 typedef unsigned long (*encode_float_proc)(double);
@@ -577,8 +578,10 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len) {
 
   if (*len ==0) return GRIB_NO_VALUES;
 
-  if((ret = grib_get_long_internal(a->parent->h,self->offsetdata,&offsetdata)) != GRIB_SUCCESS)               return ret;
-  if((ret = grib_get_long_internal(a->parent->h,self->bits_per_value,&bits_per_value)) != GRIB_SUCCESS)             return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->offsetdata,&offsetdata)) != GRIB_SUCCESS)
+	  return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->bits_per_value,&bits_per_value)) != GRIB_SUCCESS)
+	  return ret;
 
   if((ret = grib_get_long_internal(a->parent->h,self->decimal_scale_factor,&decimal_scale_factor)) != GRIB_SUCCESS)
     return ret;
@@ -589,15 +592,23 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len) {
   if((ret = grib_get_long_internal(a->parent->h,self->ieee_floats,&ieee_floats)) != GRIB_SUCCESS)
     return ret;
 
-  if((ret = grib_get_long_internal(a->parent->h,self->laplacianOperatorIsSet,&laplacianOperatorIsSet)) != GRIB_SUCCESS)                                 return ret;
-  if((ret = grib_get_double_internal(a->parent->h,self->laplacianOperator,&laplacianOperator)) != GRIB_SUCCESS)                                 return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->laplacianOperatorIsSet,&laplacianOperatorIsSet)) != GRIB_SUCCESS)
+	  return ret;
+  if((ret = grib_get_double_internal(a->parent->h,self->laplacianOperator,&laplacianOperator)) != GRIB_SUCCESS)
+	  return ret;
 
-  if((ret = grib_get_long_internal(a->parent->h,self->sub_j,&sub_j)) != GRIB_SUCCESS)                         return ret;
-  if((ret = grib_get_long_internal(a->parent->h,self->sub_k,&sub_k)) != GRIB_SUCCESS)                         return ret;
-  if((ret = grib_get_long_internal(a->parent->h,self->sub_m,&sub_m)) != GRIB_SUCCESS)                         return ret;
-  if((ret = grib_get_long_internal(a->parent->h,self->pen_j,&pen_j)) != GRIB_SUCCESS)                         return ret;
-  if((ret = grib_get_long_internal(a->parent->h,self->pen_k,&pen_k)) != GRIB_SUCCESS)                         return ret;
-  if((ret = grib_get_long_internal(a->parent->h,self->pen_m,&pen_m)) != GRIB_SUCCESS)                         return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->sub_j,&sub_j)) != GRIB_SUCCESS)
+	  return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->sub_k,&sub_k)) != GRIB_SUCCESS)
+	  return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->sub_m,&sub_m)) != GRIB_SUCCESS)
+	  return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->pen_j,&pen_j)) != GRIB_SUCCESS)
+	  return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->pen_k,&pen_k)) != GRIB_SUCCESS)
+	  return ret;
+  if((ret = grib_get_long_internal(a->parent->h,self->pen_m,&pen_m)) != GRIB_SUCCESS)
+	  return ret;
 
   self->dirty=1;
 
@@ -621,8 +632,6 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len) {
 
   Assert ( sub_j < pen_j );
 
-
-
   n_vals = (pen_j+1)*(pen_j+2);
 
   if(*len != n_vals){
@@ -713,7 +722,14 @@ printf("PACKING LAPLACE set=%ld value=%.20f\n",laplacianOperatorIsSet,laplacianO
      "unable to find nearest_smaller_value of %g for %s",min,self->reference_value);
     exit(GRIB_INTERNAL_ERROR);
   }
-  binary_scale_factor = grib_get_binary_scale_fact(max,reference_value,bits_per_value);
+  binary_scale_factor = grib_get_binary_scale_fact(max,reference_value,bits_per_value,&ret);
+  
+  if (ret==GRIB_UNDERFLOW) {
+	d=0;
+	binary_scale_factor = 0;
+	reference_value=0;
+
+  }
   s = grib_power(-binary_scale_factor,2);
 
   /* printf("D : %.30f\n",d); */
