@@ -163,6 +163,7 @@ static void init_class(grib_accessor_class* c)
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 static void init(grib_accessor* a,const long v, grib_arguments* args)
@@ -632,7 +633,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
     exit(GRIB_INTERNAL_ERROR);
   }
 
-  binary_scale_factor    = grib_get_binary_scale_fact(max,reference_value,bits_per_value);
+  binary_scale_factor    = grib_get_binary_scale_fact(max,reference_value,bits_per_value,&err);
   divisor         = grib_power(-binary_scale_factor,2);
 
 
