@@ -18,6 +18,7 @@
    IMPLEMENTS = init
    IMPLEMENTS = unpack_double
    IMPLEMENTS = pack_double
+   IMPLEMENTS = unpack_double_element
    IMPLEMENTS = value_count
    MEMBERS=const char*   type_of_compression_used
    MEMBERS=const char*   target_compression_ratio
@@ -47,6 +48,7 @@ static int unpack_double(grib_accessor*, double* val,size_t *len);
 static long value_count(grib_accessor*);
 static void init(grib_accessor*,const long, grib_arguments* );
 static void init_class(grib_accessor_class*);
+static int unpack_double_element(grib_accessor*,size_t i, double* val);
 
 typedef struct grib_accessor_data_jpeg2000_packing {
     grib_accessor          att;
@@ -114,7 +116,7 @@ static grib_accessor_class _grib_accessor_class_data_jpeg2000_packing = {
     0,      /* nearest_smaller_value */
     0,                       /* next accessor    */
     0,                    /* compare vs. another accessor   */
-    0,     /* unpack only ith value          */
+    &unpack_double_element,     /* unpack only ith value          */
     0,             		/* clear          */
 };
 
@@ -146,9 +148,9 @@ static void init_class(grib_accessor_class* c)
 	c->nearest_smaller_value	=	(*(c->super))->nearest_smaller_value;
 	c->next	=	(*(c->super))->next;
 	c->compare	=	(*(c->super))->compare;
-	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 static int first = 1;
@@ -522,3 +524,21 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 }
 
 #endif
+
+static int  unpack_double_element(grib_accessor* a, size_t idx, double* val)
+{
+	size_t size;
+	double* values;
+	int err=0;
+
+	err=grib_get_size(a->parent->h,"values",&size);
+	if (err) return err;
+	if (idx > size) return GRIB_INVALID_NEAREST;
+	
+	values=grib_context_malloc_clear(a->parent->h->context,size*sizeof(double));
+	err=grib_get_double_array(a->parent->h,"values",values,&size);
+	if (err) return err;
+	*val=values[idx];
+	grib_context_free(a->parent->h->context,values);
+	return err;
+}
