@@ -143,6 +143,7 @@ static void init_class(grib_accessor_class* c)
 	c->compare	=	(*(c->super))->compare;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 static void init(grib_accessor* a,const long v, grib_arguments* args)
@@ -191,15 +192,14 @@ static int  unpack_double_element(grib_accessor* a, size_t idx, double* val)
   size_t o = 0;
 
   n_vals = grib_value_count(a);
+  if(n_vals==0){
+    return GRIB_NOT_FOUND;
+  }
 
   if((err = grib_get_long_internal(a->parent->h,self->bits_per_value,&bits_per_value)) !=
        GRIB_SUCCESS)
     return err;
 
-  if(n_vals==0){
-    return GRIB_NOT_FOUND;
-  }
-
   self->dirty=0;
 
   if((err = grib_get_double_internal(a->parent->h,self->reference_value, &reference_value)) !=
@@ -298,6 +298,14 @@ static int  unpack_double(grib_accessor* a, double* val, size_t *len)
        GRIB_SUCCESS)
     return err;
 
+  /*/
+   * check we don't decode bpv > max(ulong) as it is 
+   * not currently supported by the algorithm 
+   */
+  if ( bits_per_value > (sizeof(long)*8) ) {
+    return GRIB_INVALID_BPV;
+  }
+
   if(self->units_factor &&
 	 (grib_get_double_internal(a->parent->h,self->units_factor,&units_factor)== GRIB_SUCCESS)) {
 		  grib_set_double_internal(a->parent->h,self->units_factor,1.0);
@@ -403,6 +411,14 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
 
   if(*len == 0) return GRIB_SUCCESS;
 
+  /*/
+   * check we don't encode bpv > max(ulong)-1 as it is 
+   * not currently supported by the algorithm 
+   */
+  if ( bits_per_value > (sizeof(long)*8-1) ) {
+    return GRIB_INVALID_BPV;
+  }
+
   self->dirty=1;
 
   max = val[0];
@@ -532,7 +548,7 @@ static int pack_double(grib_accessor* a, const double* val, size_t *len)
         exit(GRIB_INTERNAL_ERROR);
       }
 
-      binary_scale_factor = grib_get_binary_scale_fact(max,reference_value,bits_per_value);
+      binary_scale_factor = grib_get_binary_scale_fact(max,reference_value,bits_per_value,&err);
     }
   }
 
