@@ -138,6 +138,7 @@ static void init_class(grib_accessor_class* c)
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 static void init(grib_accessor* a,const long l, grib_arguments* c)
@@ -172,7 +173,8 @@ static int u2s[] =  {
 	60,      /* (0) minutes */
 	3600,    /* (1) hour    */
 	86400,   /* (2) day     */
-	-1, -1, -1, -1, -1, -1,-1,
+    2592000,   /* (3) month   */
+	-1, -1, -1, -1, -1,-1,
 	10800,   /* (10) 3 hours */
 	21600,   /* (11) 6 hours */
 	43200,   /* (12) 12 hours */
@@ -186,7 +188,7 @@ static int units_index[] = {
 int grib_g1_step_get_steps(grib_accessor* a,long* start,long* end) {
 	grib_accessor_g1step_range* self = (grib_accessor_g1step_range*)a;
 	int err = 0;
-	long p1 = 0,p2 = 0,unit = 0,timeRangeIndicator=0;
+	long p1 = 0,p2 = 0,unit = 0,timeRangeIndicator=0,timeRangeIndicatorFromStepRange=0;
 	long step_unit=1;
 	char stepType[20]={0,};
     size_t stepTypeLen=20;
@@ -209,6 +211,11 @@ int grib_g1_step_get_steps(grib_accessor* a,long* start,long* end) {
 	err = grib_get_long_internal(a->parent->h,self->timeRangeIndicator,&timeRangeIndicator);
 	if(err)  return err;
 
+	/* TODO move to the def file */
+	err = grib_get_long(a->parent->h,"timeRangeIndicatorFromStepRange",&timeRangeIndicatorFromStepRange);
+	
+	if (timeRangeIndicatorFromStepRange==10) timeRangeIndicator=timeRangeIndicatorFromStepRange;
+
     if (self->stepType) {
       err = grib_get_string_internal(a->parent->h,self->stepType,stepType,&stepTypeLen);
       if(err)  return err;
@@ -219,7 +226,7 @@ int grib_g1_step_get_steps(grib_accessor* a,long* start,long* end) {
 
 	if (timeRangeIndicator==10) *start = *end = (p1<<8) | (p2<<0);
 	else if (!strcmp(stepType,"instant")) *start = *end = p1;
-    else if (!strcmp(stepType,"precip")) {*start =0; *end = p1;}
+    else if (!strcmp(stepType,"accum") && timeRangeIndicator==0 ) {*start =0; *end = p1;}
 
 	if(*start > *end) *end = *start;
 
@@ -229,21 +236,19 @@ int grib_g1_step_get_steps(grib_accessor* a,long* start,long* end) {
 	newend   = (*end)   * u2s[unit];
 
 	if (newstart % u2s[step_unit] != 0 || newend % u2s[step_unit] != 0) {
-/*
-		*start = -1;
-		*end   = -1;
+		size_t step_unit_string_len=10;
+		char step_unit_string[10];
 		if (self->step_unit != NULL)
 			grib_get_string(a->parent->h,self->step_unit,step_unit_string,&step_unit_string_len);
 		else
 			sprintf(step_unit_string,"h");
-		*/
 
 		err = grib_set_long_internal(a->parent->h,self->step_unit,unit);
-		/*
-		grib_context_log(a->parent->h->context,GRIB_LOG_ERROR,
-				"unable to espress the step in %s",
+		
+		grib_context_log(a->parent->h->context,GRIB_LOG_WARNING,
+				"unable to represent the step in %s",
 				step_unit_string);
-		*/
+	
 	} else {
 		*start = newstart/u2s[step_unit];
 		*end   = newend/u2s[step_unit];
@@ -353,6 +358,7 @@ static int pack_string(grib_accessor* a, const char* val, size_t *len){
 	long timeRangeIndicator=0,P1=0,P2=0;
 	long start=0,end=-1,unit=0,ounit=0,step_unit=1;
 	int ret=0;
+	long end_sec,start_sec;
 	char *p=NULL,*q=NULL;
 	int instant=0;
     char stepType[20]={0,};
@@ -363,6 +369,9 @@ static int pack_string(grib_accessor* a, const char* val, size_t *len){
       if(ret)  return ret;
     } else sprintf(stepType,"unknown");
 
+	if((ret = grib_set_long_internal(h,"timeRangeIndicatorFromStepRange",-1)))
+		return ret;
+
 	/* don't change timeRangeIndicator when setting step EXCEPT IF instant*/
 	if((ret = grib_get_long_internal(h,self->timeRangeIndicator,&timeRangeIndicator)))
 		return ret;
@@ -387,15 +396,29 @@ static int pack_string(grib_accessor* a, const char* val, size_t *len){
 		ret = grib_set_long_internal(h,self->p2,end);
 		return ret;
 	}
+	end_sec=end*u2s[step_unit];
+	start_sec=start*u2s[step_unit];
 
+	if ( ( end_sec > 918000 || start_sec > 918000 ) &&
+			  	h->context->gribex_mode_on && instant ) {
+		timeRangeIndicator = 10;
+		if((ret = grib_set_long_internal(h,self->timeRangeIndicator,10)))
+			return ret;
+		/* TODO move to the def file*/
+		if((ret = grib_set_long_internal(h,"timeRangeIndicatorFromStepRange",10)))
+			return ret;
+	}
+	
 	if (timeRangeIndicator == 10) {
 		long off=0;
 		grib_accessor* p1_accessor=NULL;
-		if (end != start) {
-			grib_context_log(h->context,GRIB_LOG_ERROR,
+		if ( end != start && !h->context->gribex_mode_on) {
+			if ( h->context->gribex_mode_on==0 ) {
+				grib_context_log(h->context,GRIB_LOG_ERROR,
 					"Unable to set %s: end must be equal to start when timeRangeIndicator=10",
 					a->name);
-			return GRIB_WRONG_STEP; 
+				return GRIB_WRONG_STEP; 
+			} else start = end;
 		}
 		if ((ret=grib_g1_step_apply_units(&start,&end,&step_unit,&P1,&P2,&unit,65535,instant))
 				!=GRIB_SUCCESS) {
@@ -418,20 +441,24 @@ static int pack_string(grib_accessor* a, const char* val, size_t *len){
 		return ret;
 	}
 
-	if ((ret=grib_g1_step_apply_units(&start,&end,&step_unit,&P1,&P2,&unit,255,instant))
-			!=GRIB_SUCCESS) {
+	if ( (ret=grib_g1_step_apply_units(&start,&end,&step_unit,&P1,&P2,&unit,255,instant))
+			!=GRIB_SUCCESS ) {
 
-		if (instant) {
+		if (instant || h->context->gribex_mode_on) {
 			long off=0;
 			grib_accessor* p1_accessor=NULL;
 			if((ret = grib_set_long_internal(h,self->timeRangeIndicator,10)))
 				return ret;
-			if (end != start) {
+			/* TODO move to the def file*/
+			if((ret = grib_set_long_internal(h,"timeRangeIndicatorFromStepRange",10)))
+				return ret;
+			if (end != start && !h->context->gribex_mode_on) {
 				grib_context_log(h->context,GRIB_LOG_ERROR,
 					"Unable to set %s: end must be equal to start when timeRangeIndicator=10",
 		 																			a->name);
 				return GRIB_WRONG_STEP;
-			}
+			} else start=end;
+			
 			if ((ret=grib_g1_step_apply_units(&start,&end,&step_unit,&P1,&P2,&unit,65535,instant))
 						  !=GRIB_SUCCESS) {
 				grib_context_log(h->context,GRIB_LOG_ERROR,"unable to find units to set %s=%s",a->name,val);
