@@ -122,6 +122,7 @@ static void init_class(grib_accessor_class* c)
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 
@@ -243,10 +244,12 @@ static int pack_long(grib_accessor* a, const long* val, size_t *len)
       break;
   }
 
-  if((ret = grib_set_long_internal(a->parent->h, self->scale_first,scale_first))
-      !=GRIB_SUCCESS) return ret;
-  if((ret = grib_set_long_internal(a->parent->h, self->value_first,value_first))
-      !=GRIB_SUCCESS) return ret;
+  if ( type_first>9 ) {
+	if((ret = grib_set_long_internal(a->parent->h, self->scale_first,scale_first))
+		!=GRIB_SUCCESS) return ret;
+	if((ret = grib_set_long_internal(a->parent->h, self->value_first,value_first))
+		!=GRIB_SUCCESS) return ret;
+  }
 
   return GRIB_SUCCESS;
 }
