@@ -121,6 +121,7 @@ grib_accessor_class* grib_accessor_class_gen = &_grib_accessor_class_gen;
 static void init_class(grib_accessor_class* c)
 {
 }
+
 /* END_CLASS_IMP */
 
 static void init(grib_accessor* a,const long len, grib_arguments* param)
@@ -129,9 +130,12 @@ static void init(grib_accessor* a,const long len, grib_arguments* param)
 
  if (a->flags & GRIB_ACCESSOR_FLAG_TRANSIENT) {
     a->length = 0;
-	if (a->vvalue != NULL)
-	        grib_context_free_persistent(a->parent->h->context, a->vvalue);
-    a->vvalue = grib_context_malloc_clear_persistent(a->parent->h->context,sizeof(grib_virtual_value));
+	if (a->vvalue != NULL) {
+	        grib_context_free(a->parent->h->context, a->vvalue);
+			a->vvalue=NULL;
+	}
+
+    a->vvalue = grib_context_malloc_clear(a->parent->h->context,sizeof(grib_virtual_value));
     a->vvalue->type=grib_accessor_get_native_type(a);
     a->vvalue->length=len;
     if (act->default_value!=NULL) {
@@ -146,12 +150,12 @@ static void init(grib_accessor* a,const long len, grib_arguments* param)
        switch(type) {
          case GRIB_TYPE_DOUBLE:
            grib_expression_evaluate_double(a->parent->h,expression,&d);
-           pack_double(a,&d,&len);
+           grib_pack_double(a,&d,&len);
            break;
 
          case GRIB_TYPE_LONG:
            grib_expression_evaluate_long(a->parent->h,expression,&l);
-           pack_long(a,&l,&len);
+           grib_pack_long(a,&l,&len);
            break;
 
          default:
@@ -162,7 +166,7 @@ static void init(grib_accessor* a,const long len, grib_arguments* param)
              exit(ret);
            }
            len = strlen(p)+1;
-           pack_string(a,p,&len);
+           grib_pack_string(a,p,&len);
            break;
        }
     }
@@ -387,6 +391,7 @@ static int pack_long(grib_accessor* a, const long*  v, size_t *len){
     return ret;
   }
   grib_context_log(c,GRIB_LOG_ERROR, " Should not grib_pack %s as long", a->name);
+  Assert(0);
   return GRIB_NOT_IMPLEMENTED;
 }
 
@@ -441,6 +446,7 @@ static void destroy(grib_context* ct, grib_accessor* a)
 {
   grib_dependency_remove_observed(a);
   grib_dependency_remove_observer(a);
+  if (a->vvalue) grib_context_free(a->parent->h->context,a->vvalue);
 }
 
 static grib_section* sub_section(grib_accessor* a)
