@@ -129,6 +129,7 @@ static void init_class(grib_accessor_class* c)
 	c->unpack_double_element	=	(*(c->super))->unpack_double_element;
 	c->clear	=	(*(c->super))->clear;
 }
+
 /* END_CLASS_IMP */
 
 static void init(grib_accessor* a,const long l, grib_arguments* c)
@@ -182,12 +183,14 @@ static int  unpack_long(grib_accessor* a, long* val, size_t *len)
   if ( bpv != 0 ) {
     if((ret = grib_get_long_internal(a->parent->h, self->numberOfCodedValues,&numberOfCodedValues)) != GRIB_SUCCESS)
       return ret;
-
+#if 0
      if (*val != numberOfCodedValues) {
        grib_context_log(a->parent->h->context,GRIB_LOG_WARNING,
        "number of values from data section (%ld) doesn't match geography (%ld)",numberOfCodedValues,*val);
        /* *val=numberOfCodedValues; */
-     }
+  	}
+#endif
+			   
   }
 
 
