@@ -213,6 +213,30 @@ typedef struct grib_index grib_index;
  */
 grib_index* grib_index_new_from_file(grib_context* c,
                             char* filename,const char* keys,int *err);
+/**
+ *  Create a new index based on a set of keys.
+ *
+ * @param c           : context  (NULL for default context)
+ * @param keys        : comma separated list of keys for the index.
+ *    The type of the key can be explicitly declared appending :l for long,
+ *    :d for double, :s for string to the key name. If the type is not
+ *    declared explicitly, the native type is assumed.
+ * @param err         :  0 if OK, integer value on error
+ * @return            the newly created index
+ */
+grib_index* grib_index_new(grib_context* c, const char* keys,int *err);
+
+/**
+ *  Indexes the file given in argument in the index given in argument. 
+ *
+ * @param index       : index
+ * @param filename    : name of the file of messages to be indexed
+ * @return            0 if OK, integer value on error
+ */
+int grib_index_add_file(grib_index *index, const char *filename);
+int grib_index_write(grib_index *index, const char *filename);
+grib_index* grib_index_read(grib_context* c,const char* filename,int *err);
+
 /**
  *  Get the number of distinct values of the key in argument contained in the index. The key must belong to the index.
  *
@@ -1182,6 +1206,7 @@ int grib_get_native_type(grib_handle* h, const char* name,int* type);
 
 void grib_check(const char* call,const char*  file,int line,int e,const char* msg);
 #define GRIB_CHECK(a,msg) grib_check(#a,__FILE__,__LINE__,a,msg)
+#define GRIB_CHECK_NOLINE(a,msg) grib_check(#a,0,0,a,msg)
 
 
 int grib_set_values(grib_handle* h,grib_values*  grib_values , size_t arg_count);
@@ -1196,6 +1221,8 @@ int grib_datetime_to_julian(long year, long month, long day, long hour, long min
 long grib_julian_to_date(long jdate);
 long grib_date_to_julian(long ddate);
 
+void grib_get_reduced_row(long pl,double lon_first,double lon_last,long* npoints,long* ilon_first, long* ilon_last );
+
 /* read products */
 int wmo_read_any_from_file(FILE *f, void *buffer, size_t *len);
 int wmo_read_any_from_stream(void *stream_data, long (*stream_proc )(void *, void *buffer, long len ), void *buffer, size_t *len);
@@ -1309,4 +1336,12 @@ void *wmo_read_any_from_file_malloc(FILE *f, int *err);
 #define GRIB_CONSTANT_FIELD		-48
 /** Switch unable to find a matching case */
 #define GRIB_SWITCH_NO_MATCH		-49
+/** Underflow */
+#define GRIB_UNDERFLOW		-50
+/** Message malformed */
+#define GRIB_MESSAGE_MALFORMED		-51
+/** Index is corrupted */
+#define GRIB_CORRUPTED_INDEX		-52
+/** Invalid number of bits per value */
+#define GRIB_INVALID_BPV		-53
 #endif
