@@ -52,12 +52,14 @@ extern "C" {
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <dirent.h>
 #include <unistd.h>
 #include <errno.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <math.h>
 #include <ctype.h>
+#include <inttypes.h>
 
 #ifdef  HAVE_STRING_H
 #include <string.h>
@@ -96,12 +98,13 @@ extern int pthread_mutexattr_settype(pthread_mutexattr_t* attr,int type);
 #include "grib_api.h"
 
 #define GRIB_UNKNOWN_VALUE   -9999.999
+#define GRIB_KEY_UNDEF	"undef"
 
 #define GRIB_HANDLE_BIG_ECMWF_GRIB1    1
 
 #define MAX_FILE_HANDLES_WITH_MULTI 10
-#define MAX_NUM_ACCESSORS 1000
-#define MAX_NUM_CONCEPTS 1000
+#define ACCESSORS_ARRAY_SIZE 2000
+#define MAX_NUM_CONCEPTS 2000
 
 #define GRIB_NAMESPACE      10
 
@@ -284,7 +287,7 @@ struct grib_action
     unsigned long            flags;
     char                    *defaultkey; /** name of the key used as default if not found  */
 	grib_arguments*         default_value; /** default expression as in .def file */
-	int                     priority;     /** higher priority keys must be set before **/
+	char*					set;
 };
 
 typedef struct grib_accessor_list grib_accessor_list;
@@ -358,7 +361,7 @@ struct grib_buffer
 *  @see  grib_accessor_class
 */
 
-#define MAX_ACCESSOR_NAMES 8
+#define MAX_ACCESSOR_NAMES 20
 
 typedef struct grib_virtual_value grib_virtual_value;
 
@@ -383,6 +386,7 @@ struct grib_accessor
   grib_accessor          *previous;   /** < next accessor in list                      */
   grib_accessor_class    *cclass;     /** < behavior of the accessor                   */
   unsigned long           flags;      /** < Various flags                              */
+  grib_section*           sub_section;
 
   const char*             all_names[MAX_ACCESSOR_NAMES]  ;   /** < name of the accessor  */
   const char*             all_name_spaces[MAX_ACCESSOR_NAMES]; /** < namespace to which the accessor belongs    */
@@ -391,7 +395,7 @@ struct grib_accessor
   grib_accessor          *same;      /** < accessors with the same name */
   long                   loop;      /** < used in lists */
   grib_virtual_value*    vvalue;    /** < virtual value used when transient flag on **/
-  int                    priority;     /** higher priority keys must be set before **/
+  const char*            set;
 
 };
 
@@ -407,7 +411,7 @@ struct grib_accessor
 #define GRIB_ACCESSOR_FLAG_COPY_OK          (1<<9)
 #define GRIB_ACCESSOR_FLAG_FUNCTION         (1<<10)
 #define GRIB_ACCESSOR_FLAG_DATA             (1<<11)
-#define GRIB_ACCESSOR_FLAG_REQUIRED         (1<<12)
+#define GRIB_ACCESSOR_FLAG_NO_FAIL          (1<<12)
 #define GRIB_ACCESSOR_FLAG_TRANSIENT        (1<<13)
 #define GRIB_ACCESSOR_FLAG_STRING_TYPE      (1<<14)
 #define GRIB_ACCESSOR_FLAG_LONG_TYPE        (1<<15)
@@ -588,7 +592,7 @@ struct grib_handle
     size_t gts_header_len;
     int  use_trie;
     int  trie_invalid;
-    grib_accessor* accessors[MAX_NUM_ACCESSORS];
+    grib_accessor* accessors[ACCESSORS_ARRAY_SIZE];
     char* section_offset[MAX_NUM_SECTIONS];
     char* section_length[MAX_NUM_SECTIONS];
     int sections_count;
@@ -698,8 +702,9 @@ typedef struct grib_concept_value grib_concept_value;
 
 struct grib_concept_value {
   grib_concept_value*          next;
-  grib_concept_value_name*     names;
+  char*                        name;
   grib_concept_condition*      conditions;
+  grib_trie*		           index;
 };
 
 /* ----------*/
@@ -752,10 +757,11 @@ struct grib_context
     grib_itrie*                     concepts_index;
     int                             concepts_count;
     grib_concept_value*             concepts[MAX_NUM_CONCEPTS];
-    grib_string*                    def_files;
+    grib_trie*                      def_files;
     
     grib_string*                    blacklist;
     int                             ieee_packing;
+	grib_trie*                      classes;
 #if GRIB_PTHREADS
     pthread_mutex_t                 mutex;
 #endif
@@ -773,6 +779,7 @@ struct grib_file {
   char* mode;
   long refcount;
   grib_file* next;
+  short id;
 } ;
 
 struct grib_file_pool {
@@ -917,6 +924,8 @@ struct grib_index {
   grib_field_tree* fields;
   grib_field_list* fieldset;
   grib_field_list* current;
+  grib_file* files;
+  int count;
 };
 
 /* header compute */
@@ -1007,7 +1016,7 @@ struct grib_codetable {
   code_table_entry entries[1];
 };
 
-#if 0
+#if GRIB_TIMER
 typedef struct grib_timer {
 
     struct timeval start_;
@@ -1026,6 +1035,10 @@ typedef struct grib_timer {
 
     struct grib_timer *next_;
 } grib_timer;
+#else
+typedef struct grib_timer {
+    char nothing;
+} grib_timer;
 #endif
 
 typedef struct j2k_encode_helper {
