@@ -12,13 +12,13 @@ void grib_dump_action_tree(grib_context *ctx, FILE *out);
 void grib_xref_action_branch(FILE *out, grib_action *a, const char *path);
 
 /* action_class_alias.c */
-grib_action *grib_action_create_alias(grib_context *context, const char *name, const char *arg1, const char *name_space, int flags, int priority);
+grib_action *grib_action_create_alias(grib_context *context, const char *name, const char *arg1, const char *name_space, int flags);
 
 /* action_class_gen.c */
-grib_action *grib_action_create_gen(grib_context *context, const char *name, const char *op, const long len, grib_arguments *params, grib_arguments *default_value, int flags, const char *name_space);
+grib_action *grib_action_create_gen(grib_context *context, const char *name, const char *op, const long len, grib_arguments *params, grib_arguments *default_value, int flags, const char *name_space, const char *set);
 
 /* action_class_if.c */
-grib_action *grib_action_create_if(grib_context *context, grib_expression *expression, grib_action *block_true, grib_action *block_false);
+grib_action *grib_action_create_if(grib_context *context, grib_expression *expression, grib_action *block_true, grib_action *block_false, int transient);
 
 /* action_class_switch.c */
 grib_action *grib_action_create_switch(grib_context *context, grib_arguments *args, grib_case *Case, grib_action *Default);
@@ -59,16 +59,13 @@ grib_action *grib_action_create_trigger(grib_context *context, grib_arguments *a
 /* action_class_when.c */
 grib_action *grib_action_create_when(grib_context *context, grib_expression *expression, grib_action *block_true, grib_action *block_false);
 
-/* action_class_late_when.c */
-grib_action *grib_action_create_late_when(grib_context *context, grib_expression *expression, grib_action *block_true, grib_action *block_false);
-
 /* action_class_concept.c */
 grib_action *grib_action_create_concept(grib_context *context, const char *name, grib_concept_value *concept, const char *basename, const char *name_space, const char *defaultkey, const char *masterDir, const char *localDir, int flags, int nofail);
 const char *grib_concept_evaluate(grib_handle *h, grib_action *act);
 int grib_concept_apply(grib_handle *h, grib_action *act, const char *name);
 
 /* action_class_set.c */
-grib_action *grib_action_create_set(grib_context *context, const char *name, grib_expression *expression);
+grib_action *grib_action_create_set(grib_context *context, const char *name, grib_expression *expression, int nofail);
 
 /* action_class_set_darray.c */
 grib_action *grib_action_create_set_darray(grib_context *context, const char *name, grib_darray *darray);
@@ -108,13 +105,13 @@ int grib_unpack_string(grib_accessor *a, char *v, size_t *len);
 int grib_unpack_long(grib_accessor *a, long *v, size_t *len);
 long grib_accessor_get_native_type(grib_accessor *a);
 grib_section *grib_get_sub_section(grib_accessor *a);
+grib_section *grib_get_sub_section(grib_accessor *a);
 long grib_get_next_position_offset(grib_accessor *a);
 long grib_byte_offset(grib_accessor *a);
 long grib_byte_count(grib_accessor *a);
 long grib_value_count(grib_accessor *a);
 int grib_accessor_notify_change(grib_accessor *a, grib_accessor *changed);
 void grib_init_accessor(grib_accessor *a, const long len, grib_arguments *args);
-void grib_post_init(grib_accessor *a);
 void grib_free_accessor(grib_context *ct, grib_accessor *a);
 void grib_update_size(grib_accessor *a, size_t len);
 int grib_nearest_smaller_value(grib_accessor *a, double val, double *nearest);
@@ -125,8 +122,7 @@ int grib_compare_accessors(grib_accessor *a1, grib_accessor *a2, int compare_fla
 const char *grib_get_type_name(int type);
 
 /* grib_concept.c */
-grib_concept_value_name *grib_concept_value_name_new(grib_context *c, const char *name);
-grib_concept_value *grib_concept_value_new(grib_context *c, grib_concept_value_name *names, grib_concept_condition *conditions);
+grib_concept_value *grib_concept_value_new(grib_context *c, const char *name, grib_concept_condition *conditions);
 void grib_concept_value_delete(grib_context *c, grib_concept_value *v);
 grib_concept_condition *grib_concept_condition_new(grib_context *c, const char *name, grib_expression *expression);
 void grib_concept_condition_delete(grib_context *c, grib_concept_condition *v);
@@ -143,6 +139,9 @@ grib_iarray *grib_iarray_resize(grib_context *c, grib_iarray *v);
 grib_iarray *grib_iarray_push(grib_context *c, grib_iarray *v, long val);
 void grib_iarray_delete(grib_context *c, grib_iarray *v);
 
+/* grib_accessor_classes_hash.c */
+struct accessor_class_hash *grib_accessor_classes_hash(const char *str, unsigned int len);
+
 /* grib_accessor_class_array.c */
 
 /* grib_accessor_class_assert.c */
@@ -216,8 +215,6 @@ void grib_codetable_delete(grib_context *c);
 
 /* grib_accessor_class_when.c */
 
-/* grib_accessor_class_late_when.c */
-
 /* grib_accessor_class_g1verificationdate.c */
 
 /* grib_accessor_class_g1day_of_the_year_date.c */
@@ -256,6 +253,8 @@ int grib_g1_step_apply_units(long *start, long *end, long *step_unit, long *P1,
 
 /* grib_accessor_class_gts_header.c */
 
+/* grib_accessor_class_ifs_param.c */
+
 /* grib_accessor_class_julian_day.c */
 
 /* grib_accessor_class_latlonvalues.c */
@@ -419,6 +418,8 @@ second_order_packed *grib_get_second_order_groups(grib_context *c, const unsigne
 
 /* grib_accessor_class_gds_is_present.c */
 
+/* grib_accessor_class_select_step_template.c */
+
 /* grib_accessor_class_md5.c */
 
 /* grib_jasper_encoding.c */
@@ -441,7 +442,27 @@ grib_action *grib_action_create_set_missing(grib_context *context, const char *n
 /* grib_accessor_class_suppressed.c */
 
 /* grib_index.c */
+int grib_index_compress(grib_index *index);
+int grib_read_uchar(FILE *fh, unsigned char *val);
+int grib_read_short(FILE *fh, short *val);
+int grib_read_long(FILE *fh, long *val);
+int grib_read_unsigned_long(FILE *fh, unsigned long *val);
+int grib_write_uchar(FILE *fh, unsigned char val);
+int grib_write_short(FILE *fh, short val);
+int grib_write_long(FILE *fh, long val);
+int grib_write_unsigned_long(FILE *fh, unsigned long val);
+int grib_write_string(FILE *fh, const char *s);
+int grib_write_identifier(FILE *fh);
+int grib_write_null_marker(FILE *fh);
+int grib_write_not_null_marker(FILE *fh);
+char *grib_read_string(grib_context *c, FILE *fh, int *err);
+grib_field_tree *grib_read_field_tree(grib_context *c, FILE *fh, grib_file **files, int *err);
+grib_index *grib_index_new(grib_context *c, const char *key, int *err);
 void grib_index_delete(grib_index *index);
+int grib_index_write(grib_index *index, const char *filename);
+grib_index *grib_index_read(grib_context *c, const char *filename, int *err);
+int grib_index_search_same(grib_index *index, grib_handle *h);
+int grib_index_add_file(grib_index *index, const char *filename);
 grib_index *grib_index_new_from_file(grib_context *c, char *filename, const char *keys, int *err);
 int grib_index_get_size(grib_index *index, const char *key, size_t *size);
 int grib_index_get_string(grib_index *index, const char *key, char **values, size_t *size);
@@ -450,8 +471,12 @@ int grib_index_get_double(grib_index *index, const char *key, double *values, si
 int grib_index_select_long(grib_index *index, const char *skey, long value);
 int grib_index_select_double(grib_index *index, const char *skey, double value);
 int grib_index_select_string(grib_index *index, const char *skey, char *value);
+grib_handle *grib_index_get_handle(grib_field *field, int *err);
+void grib_index_dump(grib_index *index);
+char *grib_get_field_file(grib_index *index, off_t *offset);
 grib_handle *grib_handle_new_from_index(grib_index *index, int *err);
 void grib_index_rewind(grib_index *index);
+int grib_index_search(grib_index *index, grib_index_key *keys);
 
 /* grib_accessor_class_number_of_points_gaussian.c */
 
@@ -476,7 +501,7 @@ void grib_index_rewind(grib_index *index);
 /* grib_accessor_class_long_vector.c */
 
 /* grib_gaussian_reduced.c */
-void grib_gaussian_reduced_sub_row(long pl, double lon_first, double lon_last, long *npoints, long *ilon_first, long *ilon_last);
+void grib_get_reduced_row(long pl, double lon_first, double lon_last, long *npoints, long *ilon_first, long *ilon_last);
 
 /* grib_accessor_class_abstract_vector.c */
 
@@ -511,6 +536,27 @@ int grib_encode_unsigned_longb(unsigned char *p, unsigned long val, long *bitp,
 int grib_encode_double_array(size_t n_vals, const double *val, long bits_per_value, double reference_value, double d, double divisor, unsigned char *p, long *off);
 int grib_encode_double_array_complex(size_t n_vals, double *val, long nbits, double reference_value, double *scal, double d, double divisor, unsigned char *p, long *bitp);
 
+/* grib_timer.c */
+double proc_cpu(void);
+char *timename(double t);
+grib_timer *grib_get_timer(grib_context *c, const char *name, const char *statname, int elapsed);
+int grib_timer_start(grib_timer *t);
+int grib_timer_stop(grib_timer *t, long total);
+double grib_timer_value(grib_timer *t);
+const char *bytename(double bytes);
+void grib_timer_print(grib_timer *t);
+void grib_timer_partial_rate(grib_timer *t, double start, long total);
+void grib_print_all_timers(void);
+void grib_reset_all_timers(void);
+grib_timer *grib_get_timer(grib_context *c, const char *name, const char *statname, int elapsed);
+int grib_timer_start(grib_timer *t);
+int grib_timer_stop(grib_timer *t, long total);
+double grib_timer_value(grib_timer *t);
+void grib_timer_print(grib_timer *t);
+void grib_timer_partial_rate(grib_timer *t, double start, long total);
+void grib_print_all_timers(void);
+void grib_reset_all_timers(void);
+
 /* grib_ibmfloat.c */
 unsigned long grib_ibm_to_long(double x);
 double grib_ibmfloat_error(double x);
@@ -653,9 +699,13 @@ grib_handle *grib_fieldset_retrieve(grib_fieldset *set, int i, int *err);
 
 /* grib_filepool.c */
 void grib_file_pool_clean(void);
-grib_file *grib_file_open(char *filename, char *mode, int *err);
-void grib_file_close(char *filename, int *err);
-grib_file *grib_get_file(char *filename, int *err);
+grib_file *grib_file_pool_get_files(void);
+int grib_file_pool_read(grib_context *c, FILE *fh);
+int grib_file_pool_write(FILE *fh);
+grib_file *grib_file_open(const char *filename, const char *mode, int *err);
+void grib_file_close(const char *filename, int *err);
+grib_file *grib_get_file(const char *filename, int *err);
+grib_file *grib_find_file(short id);
 grib_file *grib_file_new(grib_context *c, const char *name, int *err);
 void grib_file_delete(grib_file *file);
 
@@ -705,6 +755,14 @@ grib_math *grib_math_clone(grib_context *c, grib_math *m);
 void grib_math_delete(grib_context *c, grib_math *m);
 grib_math *grib_math_new(grib_context *c, const char *formula, int *err);
 
+/* grib_hash_keys.c */
+struct grib_keys_hash *grib_keys_hash_get(const char *str, unsigned int len);
+grib_itrie *grib_hash_keys_new(grib_context *c, int *count);
+void grib_hash_keys_delete(grib_itrie *t);
+int grib_hash_keys_get_id(grib_itrie *t, const char *key);
+int grib_hash_keys_insert(grib_itrie *t, const char *key);
+int grib_hash_keys_get_size(grib_itrie *t);
+
 /* grib_io.c */
 int wmo_read_any_from_file(FILE *f, void *buffer, size_t *len);
 int wmo_read_grib_from_file(FILE *f, void *buffer, size_t *len);
@@ -724,6 +782,7 @@ grib_trie *grib_trie_new(grib_context *c);
 void grib_trie_delete(grib_trie *t);
 void grib_trie_clear(grib_trie *t);
 void *grib_trie_insert(grib_trie *t, const char *key, void *data);
+void *grib_trie_insert_no_replace(grib_trie *t, const char *key, void *data);
 void *grib_trie_get(grib_trie *t, const char *key);
 
 /* grib_itrie.c */
@@ -771,7 +830,7 @@ int grib_find_all_accessors(grib_handle *h, const char *name, search_all_callbac
 
 /* grib_scaling.c */
 double grib_power(long s, long n);
-long grib_get_binary_scale_fact(double max, double min, long bpval);
+long grib_get_binary_scale_fact(double max, double min, long bpval, int *ret);
 long grib_get_bits_per_value(double max, double min, long binary_scale_factor);
 long grib_get_decimal_scale_fact(double max, double min, long bpval, long binary_scale);
 
@@ -849,6 +908,9 @@ void grib_fail(const char *expr, const char *file, int line);
 /* grib_expression_class_binop.c */
 grib_expression *new_binop_expression(grib_context *c, grib_binop_long_proc long_func, grib_binop_double_proc double_func, grib_expression *left, grib_expression *right);
 
+/* grib_expression_class_true.c */
+grib_expression *new_true_expression(grib_context *c);
+
 /* grib_expression_class_string_compare.c */
 grib_expression *new_string_compare_expression(grib_context *c, grib_expression *left, grib_expression *right);
 
@@ -889,6 +951,8 @@ grib_nearest *grib_nearest_factory(grib_handle *h, grib_arguments *args);
 
 /* grib_nearest_class_sh.c */
 
+/* grib_iterator_class_polar_stereographic.c */
+
 /* grib_iterator_class_lambert_azimuthal_equal_area.c */
 
 /* grib_iterator.c */
