@@ -493,7 +493,7 @@ int grib_decode_double_array(const unsigned char* p, long *bitp, long bitsPerVal
                              double reference_value,double s,double d,
                              size_t n_vals,double* val) {
   long i=0;
-  long lvalue = 0;
+  unsigned long lvalue = 0;
 
   if(bitsPerValue%8)
   {
