@@ -12,6 +12,9 @@
 #include <stdlib.h>
 #include <unistd.h>
 
+static int file_found=1;
+static int file_not_found=2;
+
 #if MANAGE_MEM
 
 #else
@@ -79,7 +82,7 @@ static void  default_log(const grib_context* c, int level, const char* mess){
 		fprintf(stderr, "GRIB_API ERROR   :  %s\n", mess);
 		/*Assert(1==0);*/
 	}
-	if(level == GRIB_LOG_FATAL)   fprintf(stderr, "GRIB_API FATAL   :  %s\n", mess);
+	if(level == GRIB_LOG_FATAL)   fprintf(stderr, "GRIB_API ERROR   :  %s\n", mess);
 	if(level == GRIB_LOG_DEBUG)   fprintf(stderr, "GRIB_API DEBUG   :  %s\n", mess);
 	if(level == GRIB_LOG_WARNING) fprintf(stderr, "GRIB_API WARNING :  %s\n", mess);
 	if(level == GRIB_LOG_INFO)    fprintf(stderr, "GRIB_API INFO    :  %s\n", mess);
@@ -192,7 +195,8 @@ static  grib_context default_grib_context = {
 	{0,},                         /* concepts */
 	0,                            /* def_files */
 	0,                            /* ieee_packing */
-	0                             /* blacklist */
+	0,                            /* blacklist */
+    0                             /* classes */
 #if GRIB_PTHREADS
 		,PTHREAD_MUTEX_INITIALIZER     /*mutex*/
 #endif
@@ -236,11 +240,13 @@ grib_context* grib_context_get_default(){
 			default_grib_context.grib_definition_files_path = GRIB_DEFINITION_PATH ;
 #endif
 		default_grib_context.keys_count=0;
-		default_grib_context.keys=grib_itrie_new(&(default_grib_context),
+		default_grib_context.keys=grib_hash_keys_new(&(default_grib_context),
 				&(default_grib_context.keys_count));
 
 		default_grib_context.concepts_index=grib_itrie_new(&(default_grib_context),
 				&(default_grib_context.concepts_count));
+		default_grib_context.def_files=grib_trie_new(&(default_grib_context));
+		default_grib_context.classes=grib_trie_new(&(default_grib_context));
 	}
 
 
@@ -282,7 +288,8 @@ grib_context* grib_context_new(grib_context* parent)
 	c->output_log          = default_grib_context.output_log;
 	c->print               = default_grib_context.print    ;
 	c->user_data           = default_grib_context.user_data;
-
+	c->def_files           = default_grib_context.def_files;
+	
 #if GRIB_PTHREADS
 	pthread_mutexattr_settype(&attr,PTHREAD_MUTEX_RECURSIVE);
 	pthread_mutex_init(&(c->mutex),&attr);
@@ -293,29 +300,6 @@ grib_context* grib_context_new(grib_context* parent)
 	return c;
 }
 
-static int def_file_ok(grib_context* c,char* path) {
-	grib_string* def_files=c->def_files;
-	while (def_files) {
-		if (!strcmp(def_files->string,path)) return 1;
-		def_files=def_files->next;
-	}
-	return 0;
-}
-
-static void def_file_insert(grib_context* c,char* path) {
-	grib_string* def_files=c->def_files;
-	if (!c->def_files) {
-		c->def_files=
-			(grib_string*)grib_context_malloc_clear_persistent(c,sizeof(grib_string));
-		c->def_files->string=grib_context_strdup_persistent(c,path);
-	} else {
-		while (def_files->next) def_files=def_files->next;
-		def_files->next=
-			(grib_string*)grib_context_malloc_clear_persistent(c,sizeof(grib_string));
-		def_files->next->string=grib_context_strdup_persistent(c,path);
-	}
-}
-
 static int init_definition_files_dir(grib_context* c) {
 	int err=0;
 	char* path=NULL;
@@ -385,10 +369,12 @@ const char *grib_context_full_path(grib_context* c,
 			dir=c->grib_definition_files_dir;
 
 			while (dir) {
+				int *found=NULL;
 				sprintf(full,"%s/%s",dir->string,basename);
-				if (def_file_ok(c,full)) return full;
+				if ((found=grib_trie_get(c->def_files,full))!=NULL) 
+					return *found != file_found ? NULL : full;
 				if (!access(full,F_OK)) {
-					def_file_insert(c,full);
+					grib_trie_insert(c->def_files,full,&file_found);
 					grib_context_log(c,GRIB_LOG_DEBUG,"Found def file %s",full);
 					GRIB_MUTEX_UNLOCK(&(c->mutex));
 					return full;
@@ -396,23 +382,15 @@ const char *grib_context_full_path(grib_context* c,
 				dir=dir->next;
 			}
 
-#if 0
-			/* Store missing files so we don't check for them again and again */
-			dir=c->grib_definition_files_dir;
-			sprintf(full,"%s/%s",dir->string,basename);
-			def_file_insert(c,full);
-			GRIB_MUTEX_UNLOCK(&(c->mutex));
-			return full;
-#endif
-
-
-
 		}
 
 	}
 
+	/* Store missing files so we don't check for them again and again */
+	grib_trie_insert(c->def_files,full,&file_not_found);
 	/*grib_context_log(c,GRIB_LOG_ERROR,"Def file \"%s\" not found",basename);*/
 	GRIB_MUTEX_UNLOCK(&(c->mutex));
+	full[0]=0;
 	return NULL;
 }
 
@@ -468,16 +446,11 @@ void grib_context_reset(grib_context* c){
 }
 
 void grib_context_delete( grib_context* c){
-	grib_string* def_files;
 	if (!c) c=grib_context_get_default();
-	/*  grib_associative_array_destroy(c->files);*/
-	grib_itrie_delete( c->keys);
-	while (c->def_files) {
-		def_files=c->def_files->next;
-		grib_context_free(c,c->def_files->string);
-		grib_context_free(c, c->def_files);
-		c->def_files=def_files;
-	}
+	
+	grib_hash_keys_delete( c->keys);
+	grib_trie_delete(c->def_files);
+
 	grib_context_reset( c );
 	if(c != &default_grib_context)
 		grib_context_free_persistent(&default_grib_context,c);
@@ -487,7 +460,7 @@ void* grib_context_malloc_persistent(const grib_context* c, size_t size){
 	void* p =  c->alloc_persistent_mem(c,size);
 	if(!p) {
 		grib_context_log(c,GRIB_LOG_ERROR,"grib_context_malloc: error allocating %d bytes",size);
-		exit(1);
+		Assert(1);
 	}
 	return p;
 }
