@@ -410,7 +410,8 @@ static void dump_section(grib_dumper* d,grib_accessor* a,grib_block_of_accessors
 {
   grib_dumper_debug *self = (grib_dumper_debug*)d;
   int i;
-  grib_section* s = grib_get_sub_section(a);
+  /* grib_section* s = grib_get_sub_section(a); */
+  grib_section* s = a->sub_section;
 
 
 #if 1
