@@ -269,7 +269,10 @@ grib_fieldset* grib_fieldset_new_from_files(grib_context* c,char* filenames[],
    *err=GRIB_SUCCESS;
    for (i=0;i<nfiles;i++) {
      ret=grib_fieldset_add(set,filenames[i]);
-     if (ret != GRIB_SUCCESS) *err=ret;
+     if (ret != GRIB_SUCCESS) {
+       *err=ret;
+       return NULL;
+     }
    }
 
    if (where_string) grib_fieldset_apply_where(set,where_string);
@@ -594,7 +597,7 @@ int grib_fieldset_add(grib_fieldset* set,char* filename) {
    file=grib_file_open(filename,"r",&err);
    if (!file || !file->handle) return err;
 
-   while((h = grib_handle_new_from_file(c,file->handle,&err))
+   while((h = grib_handle_new_from_file(c,file->handle,&ret))
          != NULL || ret != GRIB_SUCCESS ) {
      if (!h) return ret;
 
