@@ -41,7 +41,8 @@ static void update_sections ( grib_section *s,grib_handle* h,long offset )
 	while ( a )
 	{
 		a->offset += offset;
-		update_sections ( grib_get_sub_section ( a ),h,offset );
+		/* update_sections ( grib_get_sub_section ( a ),h,offset ); */
+		update_sections ( a->sub_section,h,offset );
 		a = a->next;
 	}
 }
@@ -113,15 +114,16 @@ int grib_handle_delete ( grib_handle* h )
 
 		Assert ( h->kid == NULL );
 
-		grib_buffer_delete ( ct,h->buffer );
-		grib_section_delete ( ct,h->root );
-
 		while ( d )
 		{
 			n = d->next;
 			grib_context_free ( ct,d );
 			d = n;
 		}
+		h->dependencies=0;
+
+		grib_buffer_delete ( ct,h->buffer );
+		grib_section_delete ( ct,h->root );
 
 		grib_context_log ( ct,GRIB_LOG_DEBUG,"grib_handle_delete: deleting handle %p",h );
 		grib_context_free ( ct,h );
