@@ -7,12 +7,7 @@
  * See LICENSE and gpl-3.0.txt for details.
  */
 
-/***************************************************************************
- *   Jean Baptiste Filippi - 01.11.2005
- *   Enrico Fucile
- *                                                                         *
- ***************************************************************************/
-#include <assert.h> 
+#include <assert.h>
 #include "grib_api_internal.h"
 
 
@@ -77,11 +72,12 @@ static int read_the_rest(reader* r,size_t message_length,unsigned char* tmp, int
 	return GRIB_SUCCESS;
 }
 
-#define CHECK_TMP_SIZE(a) if(sizeof(tmp)<(a)) { fprintf(stderr,"%s:%d sizeof(tmp)<%s %d<%d\n", __FILE__,__LINE__,#a,sizeof(tmp),a); return GRIB_INTERNAL_ARRAY_TOO_SMALL; }
+#define CHECK_TMP_SIZE(a) if(sizeof(tmp)<(a)) { fprintf(stderr,"%s:%d sizeof(tmp)<%s %d<%d\n", __FILE__,__LINE__,#a,(int)sizeof(tmp),(int)(a)); return GRIB_INTERNAL_ARRAY_TOO_SMALL; }
 
 static int read_GRIB(reader* r)
 {
-	unsigned char tmp[16368]; /* Should be enough */
+	/* unsigned char tmp[16368];  */
+	unsigned char tmp[700368]; /* Should be enough */
 	size_t length = 0;
 	long edition = 0;
 	int  err = 0;
