@@ -75,7 +75,7 @@ int grib_recompose_name(grib_handle* h, grib_accessor *observer, const char* una
 					if (!fail) {
 						sprintf(val,"undef");
 					} else {
-						grib_context_log(h->context, GRIB_LOG_WARNING,"grib_recompose_name: Problem to recompose filename with : %s, no accessor found", loc);
+						grib_context_log(h->context, GRIB_LOG_WARNING,"grib_recompose_name: Problem to recompose filename with : %s ( %s no accessor found)", uname,loc);
 						return GRIB_NOT_FOUND;
 					}
 				} else {
