@@ -114,99 +114,168 @@ int grib_set_double_internal(grib_handle* h, const char* name, double val) {
   return GRIB_NOT_FOUND;
 }
 
+typedef struct grib_key_err grib_key_err;
+struct grib_key_err {
+	char* name;
+	int err;
+	grib_key_err* next;
+};
 
 int grib_copy_namespace(grib_handle* dest, const char* name, grib_handle* src) {
-  const char* key=NULL;
-  int err=0;
-  int type;
-  size_t len;
-  char *sval = NULL;
-  unsigned char *uval = NULL;
-  double *dval = NULL;
-  long *lval = NULL;
+	int *err=0;
+	int type;
+	size_t len;
+	char *sval = NULL;
+	unsigned char *uval = NULL;
+	double *dval = NULL;
+	long *lval = NULL;
+	grib_key_err* key_err=NULL;
+	grib_key_err* first=NULL;
+	int todo=1,count=0;
+	
 
-  grib_keys_iterator* iter=NULL;
+	grib_keys_iterator* iter=NULL;
 
-  if (!dest || !src) return GRIB_NULL_HANDLE;
+	if (!dest || !src) return GRIB_NULL_HANDLE;
+	
+	iter=grib_keys_iterator_new(src,0,(char*)name);
+	
+	if (!iter) {
+		printf("ERROR: unable to get iterator for %s\n",name );
+		exit(1);
+	}
   
-  iter=grib_keys_iterator_new(src,0,(char*)name);
-  
-  if (!iter) {
-    printf("ERROR: unable to get iterator for %s\n",name );
-    exit(1);
-  }
+	while(grib_keys_iterator_next(iter))  {
+		grib_key_err* k=grib_context_malloc_clear(src->context,sizeof(grib_key_err));
+		k->err=GRIB_NOT_FOUND;
+		k->name=grib_context_strdup(src->context,grib_keys_iterator_get_name(iter));
+		if (key_err==NULL) {
+			key_err=k;
+			first=k;
+		} else {
+			key_err->next=k;
+			key_err=key_err->next;
+		}
+	}
+	
+	count=0;
+	todo=1;
+	while (todo && count<4) {
+		grib_accessor* a=NULL;
+		key_err=first;
+		while(key_err)  {
+			char* key=key_err->name;
+			err=&(key_err->err);
 
-  while(grib_keys_iterator_next(iter))  {
-    key=grib_keys_iterator_get_name(iter);
+			if (*err==GRIB_SUCCESS) {
+				key_err=key_err->next;
+				continue;
+			}
+			
+			if ((a=grib_find_accessor(dest,key))==NULL) {
+				key_err->err=GRIB_NOT_FOUND;
+				key_err=key_err->next;
+				continue;
+			}
 
-    if ( grib_is_missing(src,key,&err) && err == 0 && (err=grib_set_missing(dest,key))) {
-      if (err!=GRIB_SUCCESS) return err;
-      continue;
-    }
-  
-    if ((err=grib_get_native_type(src,key,&type))!=GRIB_SUCCESS) return err;
-  
-    if((err = grib_get_size(src,key,&len)) != GRIB_SUCCESS)  return err;
+			if (a->flags && GRIB_ACCESSOR_FLAG_READ_ONLY) {
+				key_err->err=GRIB_SUCCESS;
+				key_err=key_err->next;
+				continue;
+			}
+			
+			if ( grib_is_missing(src,key,err) && *err == 0 && (*err=grib_set_missing(dest,key))) {
+				if ( *err!=GRIB_SUCCESS && *err!=GRIB_NOT_FOUND) return *err;
+				key_err=key_err->next;
+				continue;
+			}
+		
+			if ((*err=grib_get_native_type(dest,key,&type))!=GRIB_SUCCESS) {
+				if ( *err!=GRIB_SUCCESS && *err!=GRIB_NOT_FOUND) return *err;
+				key_err=key_err->next;
+				continue;
+			}
+		
+			if((*err = grib_get_size(src,key,&len)) != GRIB_SUCCESS)  return *err;
 
-    switch (type) {
-      case GRIB_TYPE_STRING:
-        len=512;
-        sval = grib_context_malloc(src->context,len*sizeof(char));
-        
-        if((err = grib_get_string(src,key,sval,&len)) != GRIB_SUCCESS)
-          return err;
+			switch (type) {
+				case GRIB_TYPE_STRING:
+					len=512;
+					sval = grib_context_malloc(src->context,len*sizeof(char));
+					
+					if((*err = grib_get_string(src,key,sval,&len)) != GRIB_SUCCESS)
+						return *err;
 
-        if((err = grib_set_string(dest,key,sval,&len)) != GRIB_SUCCESS)
-          return err;
+					if((*err = grib_set_string(dest,key,sval,&len)) != GRIB_SUCCESS)
+						return *err;
 
-        grib_context_free(src->context,sval);
-        break;
+					grib_context_free(src->context,sval);
+					break;
 
-      case GRIB_TYPE_LONG:
-        lval = grib_context_malloc(src->context,len*sizeof(long));
+				case GRIB_TYPE_LONG:
+					lval = grib_context_malloc(src->context,len*sizeof(long));
 
-        if((err = grib_get_long_array(src,key,lval,&len)) != GRIB_SUCCESS)
-          return err;
+					if((*err = grib_get_long_array(src,key,lval,&len)) != GRIB_SUCCESS)
+						return *err;
 
-        if((err = grib_set_long_array(dest,key,lval,len)) != GRIB_SUCCESS)
-          return err;
+					if((*err = grib_set_long_array(dest,key,lval,len)) != GRIB_SUCCESS)
+						return *err;
 
-        grib_context_free(src->context,lval);
-        break;
+					grib_context_free(src->context,lval);
+					break;
 
-      case GRIB_TYPE_DOUBLE:
-        dval = grib_context_malloc(src->context,len*sizeof(double));
+				case GRIB_TYPE_DOUBLE:
+					dval = grib_context_malloc(src->context,len*sizeof(double));
 
-        if((err = grib_get_double_array(src,key,dval,&len)) != GRIB_SUCCESS)
-          return err;
+					if((*err = grib_get_double_array(src,key,dval,&len)) != GRIB_SUCCESS)
+						return *err;
 
-        if((err = grib_set_double_array(dest,key,dval,len)) != GRIB_SUCCESS)
-          return err;
+					if((*err = grib_set_double_array(dest,key,dval,len)) != GRIB_SUCCESS)
+						return *err;
 
-        break;
+					break;
 
-      case GRIB_TYPE_BYTES:
-        if (len==0) len=512;
-        uval = grib_context_malloc(src->context,len*sizeof(unsigned char));
-        
-        if((err = grib_get_bytes(src,key,uval,&len)) != GRIB_SUCCESS)
-          return err;
+				case GRIB_TYPE_BYTES:
+					if (len==0) len=512;
+					uval = grib_context_malloc(src->context,len*sizeof(unsigned char));
+					
+					if((*err = grib_get_bytes(src,key,uval,&len)) != GRIB_SUCCESS)
+						return *err;
 
-        if((err = grib_get_bytes(dest,key,uval,&len)) != GRIB_SUCCESS)
-          return err;
+					if((*err = grib_get_bytes(dest,key,uval,&len)) != GRIB_SUCCESS)
+						return *err;
 
-        grib_context_free(src->context,uval);
+					grib_context_free(src->context,uval);
 
-        break;
+					break;
 
-      default:
-        break;
-    }
+				default:
+					break;
+			}
+			key_err=key_err->next;
 
-  }
-  grib_keys_iterator_delete(iter);
-  
-  return err;
+		}
+		count++;
+		key_err=first;
+		todo=0;
+		while(key_err)  {
+			if (key_err->err==GRIB_NOT_FOUND) {
+				todo=1;
+				break;
+			}
+			key_err=key_err->next;
+		}
+	}
+	grib_keys_iterator_delete(iter);
+	key_err=first;
+	while (key_err) {
+		grib_key_err* next=key_err->next;
+		grib_context_free(src->context,key_err->name);
+		grib_context_free(src->context,key_err);
+		key_err=next;
+	}
+	
+	return *err;
 }
 
 
@@ -534,10 +603,12 @@ int grib_get_long_internal(grib_handle* h, const char* name, long* val)
 {
   int ret = grib_get_long(h,name,val);
 
-  if(ret!=GRIB_SUCCESS)
+  if(ret!=GRIB_SUCCESS) {
     grib_context_log(h->context,GRIB_LOG_ERROR,
                      "unable to get %s as long (%s)",
                      name, grib_get_error_message(ret));
+	Assert(0);
+  }
 
   return ret;
 }
