@@ -11,6 +11,6 @@
 
 for file in ${data_dir}/*.grib2 ${data_dir}/*.grib1
 do
-${tools_dir}grib_debug $file 2> /dev/null > /dev/null
+${tools_dir}grib_dump -O $file 2> /dev/null > /dev/null
 done
 
