@@ -44,7 +44,19 @@ then
   exit 1
 fi
 
-${tools_dir}grib_compare -A1.0e-8 -c values ${file}.grib1 ${file}.grib2 2> /dev/null > /dev/null
+#${tools_dir}grib_compare -A1.0e-8 -c values ${file}.grib1 ${file}.grib2 2> /dev/null > /dev/null
+${tools_dir}grib_compare -P -c values ${file}.grib1 ${file}.grib2 2> /dev/null > /dev/null
 
 done
 
+#sed "s:toolsdir:${tools_dir}:" ${tools_dir}grib1to2.txt > ${tools_dir}grib1to2.test
+#chmod +x ${tools_dir}grib1to2.test
+#${tools_dir}grib1to2.test -f ${data_dir}/test.grib1 ${data_dir}/test.grib2
+#${tools_dir}grib_get -p typeOfProcessedData:s,type:s ${data_dir}/test.grib2 > ${data_dir}/typeOfProcessedData.log
+
+#diff ${data_dir}/typeOfProcessedData.log ${data_dir}/typeOfProcessedData.ok
+
+#${tools_dir}grib_compare -Pe ${data_dir}/test.grib1 ${data_dir}/test.grib2
+
+#rm -f ${data_dir}/test.grib2
+
