@@ -9,7 +9,7 @@
 
 . ./include.sh
 
-tmpdata=/tmp/grib_api.grib
+tmpdata=grib_api.grib
 
 rm -f $tmpdata || true
 
