@@ -25,8 +25,13 @@ int main(int argc,char* argv[]) {
   outfile=argv[2];
 
   printf("indexing...\n");
-  /* Create an index from a grib file with a given set of keys*/
-  index=grib_index_new_from_file(0,infile,"shortName,level,number,step",&ret);
+
+  /* Create an index given set of keys*/
+  index=grib_index_new(0,"shortName,level,number,step",&ret);
+  if (ret) {printf("error: %s\n",grib_get_error_message(ret)); exit(ret);}
+
+  /* Indexes a file */
+  ret=grib_index_add_file(index,infile);
   if (ret) {printf("error: %s\n",grib_get_error_message(ret)); exit(ret);}
   printf("end indexing...\n");
 
@@ -115,6 +120,9 @@ int main(int argc,char* argv[]) {
   }
   printf("  %d messages selected\n",count);
 
+  grib_index_write(index,"out.gribidx");
+
+
   return 0;
 }
 
