@@ -25,7 +25,7 @@ ${tools_dir}grib_get -P count $infile >> log
  
 
 diff log ls.log 
-rm -f log
+#rm -f log
 
 cd $workdir
 
