@@ -38,7 +38,7 @@ rm -f ${data_dir}/out.grib | true
 
 diff ${data_dir}/log ${data_dir}/step.log
 
-${tools_dir}grib_filter ${data_dir}/step_grib1.filter ${data_dir}/timeRangeIndicator_0.grib > ${data_dir}/log
+(${tools_dir}grib_filter ${data_dir}/step_grib1.filter ${data_dir}/timeRangeIndicator_0.grib > ${data_dir}/log) 2>/dev/null
 
 diff ${data_dir}/log ${data_dir}/step_grib1.log
 
