@@ -7,8 +7,8 @@
 # See LICENSE and gpl-3.0.txt for details.
 
 # --- test grib edition 2 to 1 conversions with tigge data
-# ---   tests some keys remain the same after the conversion
-# ---   also test the values
+# ---   using the new features of grib_compare that allows
+# ---   namespace comparison
 
 . ./include.sh
 
@@ -17,9 +17,9 @@
 dir="${data_dir}/tigge/"
 temp="temp.grib1_"
 
-# --- exclude babj as the way they encode grib 2 makes it
-# ---   problematic to convert to grib 1
-exclusion_pattern="nothing"
+# --- Do I want to exclude any file pattern from
+# --- comparison ?
+exclusion_pattern="tcw|ssr|str|skt|cap|ci|ttr|st|tp"
 
 for file in `ls ${dir}/tigge_*.grib`
 do
@@ -32,16 +32,12 @@ if [ -z "$exclude" ]; then
 
 	${tools_dir}grib_set -s editionNumber=1 ${file} ${temp} 2> /dev/null > /dev/null
 
-
-	grib1Statistics=`${tools_dir}grib_get -p numberOfValues,numberOfPoints,max,min,average,numberOfMissing ${temp}` 
-	grib2Statistics=`${tools_dir}grib_get -p numberOfValues,numberOfPoints,max,min,average,numberOfMissing ${file}` 
-
-	if [ "$grib1Statistics" != "$grib2Statistics" ]
-	then 
-	  exit 1
-	fi
-
-	${tools_dir}grib_compare -P -c values ${temp} ${file} 2> /dev/null > /dev/null
+    # comparing on the data and geography namespace 
+    # wanted to do directly -e for edition independent comparison, 
+    # but had some problems with the mars namespace, and trying to 
+    # blacklist it while -e was on generated a segmentation fault
+    # in grib_compare
+	${tools_dir}grib_compare -P -c data:n,geography:n ${temp} ${file} 2> /dev/null > /dev/null
 
 	rm -f ${temp} || true
 fi
