@@ -113,15 +113,17 @@ grib_option grib_options[]={
 /*  {id, args, help}, on, command_line, value*/
     {"r",0,"Compare files in which the messages are not in the same order. This option is time expensive.\n",0,1,0},
     {"b:",0,0,0,1,0},
-    {"e",0,"edition independent compare. It is used to compare grib edition 1 and 2.\n",0,1,0},
+    {"e",0,"Edition independent compare. It is used to compare grib edition 1 and 2.\n",0,1,0},
     {"c:",0,0,0,1,0},
+    {"S:","start","First field to be processed.\n",0,1,0},
+    {"E:","end","Last field to be processed.\n",0,1,0},
     {"a",0,"-c option modifier. The keys listed with the option -c will be added to the list of keys compared without -c.\n"
         ,0,1,0},
     {"H",0,"Compare only message headers. Bit-by-bit compare on. Incompatible with -c option.\n",0,1,0},
     {"R:",0,0,0,1,0},
     {"A:",0,0,0,1,0},
     {"P",0,"Compare data values using the packing error as tolerance.\n",0,1,0},
-    {"T:","factor","Compare data values using factor multipied by the tolerance specified in options -P -R -A.\n",0,1,0},
+    {"T:","factor","Compare data values using factor multiplied by the tolerance specified in options -P -R -A.\n",0,1,0},
     {"w:",0,0,0,1,0},
     {"f",0,0,0,1,0},
     {"F",0,0,1,0,0},
@@ -134,6 +136,9 @@ grib_option grib_options[]={
 };
 
 grib_handle* h1=NULL;
+int counter=0;
+int start=-1;
+int end=-1;
 
 char* grib_tool_description=
          "Compare grib messages contained in two files."
@@ -160,6 +165,11 @@ int grib_tool_init(grib_runtime_options* options) {
   grib_context* context=grib_context_get_default();
 
   options->strict=1;
+  if (grib_options_on("S:")) 
+      start=atoi(grib_options_get_option("S:"));
+
+  if (grib_options_on("E:")) 
+      end=atoi(grib_options_get_option("E:"));
 
   if (grib_options_on("f")) force=1;
   else force=0;
@@ -258,7 +268,10 @@ int grib_tool_init(grib_runtime_options* options) {
   return 0;
 }
 
-int grib_tool_new_file_action(grib_runtime_options* options) {
+int grib_tool_new_filename_action(grib_runtime_options* options,const char* file) {
+   return 0;
+}
+int grib_tool_new_file_action(grib_runtime_options* options,grib_tools_file* file) {
    return 0;
 }
 
@@ -297,15 +310,68 @@ static void printInfo(grib_handle* h) {
 
 }
 
+static void print_index_key_values(grib_index* index,int counter) {
+	grib_index_key* keys=index->keys;
+	printf("== %d == ",counter);
+	while (keys) {
+		printf("%s=%s ",keys->name,keys->value);
+		keys=keys->next;
+	}
+	printf("\n");
+}
 int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h) {
   int err=0;
   count++;
-  if (options->random) 
+
+  if (options->through_index) {
+	grib_index* idx1=options->index1;
+	verbose=0;
+	counter++;
+
+	if ( start>0 && counter < start ) return 0;
+	if ( end>0 && counter > end ) { 
+		options->stop=1;
+		return 0;
+	}
+
+	grib_index_search_same(idx1,h);
+	h1=grib_handle_new_from_index(idx1,&err);
+	if (options->verbose) {
+		off_t offset=0;
+		char* filename=grib_get_field_file(options->index2,&offset);
+		printf("file1=\"%s\" ",filename);
+		filename=grib_get_field_file(options->index1,&offset);
+		printf("file2=\"%s\" \n",filename);
+		print_index_key_values(options->index1,counter);
+	}
+
+	if (!h1) {
+		if (!options->verbose)
+			print_index_key_values(idx1,counter);
+		printf("====== NOT FOUND in %s\n",options->infile->name);
+	}
+
+	if (!h1 || err!= GRIB_SUCCESS ) {
+		morein1++;
+		if (h1) grib_handle_delete(h1);
+		return 0;
+    }
+
+    if(compare_handles(h,h1,options)) {
+		error++;
+		if (!force) exit(1);
+    }
+
+	grib_handle_delete(h1);
+
+	return 0;
+	
+  } else if (options->random) 
     h1 = grib_fieldset_next_handle(options->idx,&err);
   else
     h1=grib_handle_new_from_file(h->context,options->infile_extra->file,&err);
   
-  if (!h1 || err!= GRIB_SUCCESS) {
+  if (!h1 || err!= GRIB_SUCCESS ) {
     morein2++;
     if (h1) grib_handle_delete(h1);
     return 0;
@@ -323,14 +389,20 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h) {
 
 int grib_tool_skip_handle(grib_runtime_options* options, grib_handle* h) {
   int err=0;
-  h1=grib_handle_new_from_file(h->context,options->infile_extra->file,&err);
+  if (!options->through_index && !options->random)  {
+	  h1=grib_handle_new_from_file(h->context,options->infile_extra->file,&err);
   
-  if (!h1 || err!= GRIB_SUCCESS) 
-    morein2++;
+	  if (!h1 || err!= GRIB_SUCCESS) 
+		morein2++;
 
-  grib_handle_delete(h1);
+	  grib_handle_delete(h1);
+	  
+	  
+  }
+  
   grib_handle_delete(h);
   count++;
+  
   return 0;
 }
 
@@ -343,6 +415,8 @@ int grib_tool_finalise_action(grib_runtime_options* options) {
   int err=0;
   grib_context* c=grib_context_get_default();
   error+=morein1+morein2;
+
+  /*if (grib_options_on("w:")) return 0;*/
   
   if (error) {
     printf("\n## ERRORS SUMMARY #######\n");
@@ -372,6 +446,11 @@ int grib_tool_finalise_action(grib_runtime_options* options) {
     
     printf("##\n## %d different messages out of %d\n\n",error,count);
   }
+  if (options->through_index) {
+  	grib_index_delete(options->index1);
+  	grib_index_delete(options->index2);
+  }
+
   
   if (error !=0) exit(1);
   return 0;
@@ -501,6 +580,7 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
     return err;
   }
 
+  /*
   if(len1 != len2 && type1 != GRIB_TYPE_STRING)
   {
     printInfo(h1);
@@ -508,6 +588,7 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
     save_error(c,name);
     return GRIB_COUNT_MISMATCH;
   }
+  */
 
   isMissing1= ( (grib_is_missing(h1,name,&err1)==1) && (err1 == 0) ) ? 1 : 0;
   isMissing2= ( (grib_is_missing(h2,name,&err2)==1) && (err2 == 0) ) ? 1 : 0;
@@ -618,8 +699,8 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
             printf("long [%s]: [%ld] != [%ld]\n",
                    name,*lval1,*lval2);
           else
-            printf("long [%s] %d out of %d different\n",
-                   name,countdiff,len1);
+            printf("long [%s] %d out of %ld different\n",
+                   name,countdiff,(long)len1);
         }
       }
 
@@ -724,7 +805,7 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
           printInfo(h1);
           save_error(c,name);
           if (len1>1) {
-            printf("double [%s]: %d out of %d different, ",name,countdiff,len1);
+            printf("double [%s]: %d out of %ld different, ",name,countdiff,(long)len1);
             if (compareAbsolute) printf(" max");
             printf(" absolute diff. = %g,",fabs(dval1[imaxdiff]-dval2[imaxdiff]));
             if (!compareAbsolute) printf(" max");
@@ -856,8 +937,8 @@ static int compare_handles(grib_handle* h1,grib_handle* h2,grib_runtime_options*
     const void *msg1=NULL,*msg2=NULL;
     size_t size1=0,size2=0;
     grib_handle *h11, *h22;
-    GRIB_CHECK(grib_get_message_headers(h1,&msg1,&size1),0);
-    GRIB_CHECK(grib_get_message_headers(h2,&msg2,&size2),0);
+    GRIB_CHECK_NOLINE(grib_get_message_headers(h1,&msg1,&size1),0);
+    GRIB_CHECK_NOLINE(grib_get_message_headers(h2,&msg2,&size2),0);
     if (size1==size2 && !memcmp(msg1,msg2,size1))
       return 0;
 
@@ -890,7 +971,7 @@ static int compare_handles(grib_handle* h1,grib_handle* h2,grib_runtime_options*
 
   if ( listFromCommandLine && onlyListed ) {
     for (i=0; i< options->compare_count; i++) {
-      if (blacklisted(name)) continue;
+	  if (blacklisted((char*)options->compare[i].name)) continue;
       if (options->compare[i].type == GRIB_NAMESPACE) {
         iter=grib_keys_iterator_new(h1,0,(char*)options->compare[i].name);
         if (!iter) {
@@ -914,8 +995,8 @@ static int compare_handles(grib_handle* h1,grib_handle* h2,grib_runtime_options*
   } else {
     const void *msg1=NULL,*msg2=NULL;
     size_t size1=0,size2=0;
-    GRIB_CHECK(grib_get_message(h1,&msg1,&size1),0);
-    GRIB_CHECK(grib_get_message(h2,&msg2,&size2),0);
+    GRIB_CHECK_NOLINE(grib_get_message(h1,&msg1,&size1),0);
+    GRIB_CHECK_NOLINE(grib_get_message(h2,&msg2,&size2),0);
     if (size1==size2 && !memcmp(msg1,msg2,size1))
        return 0;
 
