@@ -252,7 +252,7 @@ int grib_process_runtime_options(grib_context* context,int argc,char** argv,grib
   }
   
   options->requested_print_keys_count=MAX_KEYS;
-  GRIB_CHECK(grib_options_get_keys(argv[0],karg,0,GRIB_TYPE_UNDEFINED,
+  GRIB_CHECK_NOLINE(grib_options_get_keys(argv[0],karg,0,GRIB_TYPE_UNDEFINED,
       options->requested_print_keys,&(options->requested_print_keys_count)),0);
 
 
