@@ -28,14 +28,14 @@ GRIB_INLINE static int grib_inline_strcmp(const char* a,const char* b) {
 
 static void grib_print_header(grib_runtime_options* options,grib_handle* h);
 static void grib_tools_set_print_keys(grib_runtime_options* options,grib_handle* h,const char* ns);
-static int grib_tool_with_orderby();
-static int grib_tool_without_orderby();
+static int grib_tool_with_orderby(grib_runtime_options* options);
+static int grib_tool_without_orderby(grib_runtime_options* options);
+static int grib_tool_onlyfiles(grib_runtime_options* options);
+static int grib_tool_index(grib_runtime_options* options);
+static int process(grib_context* c,grib_runtime_options* options,const char* path);
+static int scan(grib_context* c,grib_runtime_options* options,const char* dir);
 
-int grib_tool(int argc, char **argv)
-{
-	grib_context* c=NULL;
-
-	grib_runtime_options options={
+grib_runtime_options options={
 		0,         /* verbose       */
 		0,         /* fail          */
 		0,         /* skip          */
@@ -88,21 +88,50 @@ int grib_tool(int argc, char **argv)
 		0,         /* index */
 		0,         /* random */
         0,         /* format */
+        0,         /* onlyfiles */
         0,         /* tolerance_count  */
+		0,			/* through_index */
+        0,         /* index1  */
+        0,         /* index2  */
+        0,         /* context  */
+        0,         /* stop  */
         {{0,},}    /* grib_values tolerance[MAX_KEYS] */
 
 	};
 
+static int is_index_file(const char* filename) {
+	FILE* fh;
+	char buf[8]={0,};
+	char* str="GRBIDX";
+	int ret=0;
+
+	fh=fopen(filename,"r");
+	if (!fh) return 0;
+
+	fread(buf,1,1,fh);
+	fread(buf,6,1,fh);
+
+	ret=!strcmp(buf,str);
+
+	fclose(fh);
+
+	return ret;
+}
+
+int grib_tool(int argc, char **argv)
+{
+	int ret=0;
+	grib_context* c=grib_context_get_default();
+	options.context=c;
+
 	if (getenv("DOXYGEN_USAGE") && argc==1 ) usage_doxygen();
 
-	c=grib_context_get_default();
 	grib_get_runtime_options(argc,argv,&options);
 
 	grib_tool_before_getopt(&options);
 
 	grib_process_runtime_options(c,argc,argv,&options);
 
-
 	grib_tool_init(&options);
 	if (options.dump_filename) {
 		dump_file= fopen(options.dump_filename,"w");
@@ -114,12 +143,24 @@ int grib_tool(int argc, char **argv)
 		dump_file=stdout;
 	}
 
-	if (options.orderby)
-		return grib_tool_with_orderby(&options);
-	else
-		return grib_tool_without_orderby(&options);
+	if (is_index_file(options.infile->name) && 
+		( options.infile_extra && is_index_file(options.infile_extra->name))) { 
+			options.through_index=1;
+			return grib_tool_index(&options);
+		}
+
+	if (options.onlyfiles) 
+		ret=grib_tool_onlyfiles(&options);
+	else {
+		if (options.orderby)
+			ret=grib_tool_with_orderby(&options);
+		else
+			ret=grib_tool_without_orderby(&options);
+	}
 
 	if (options.dump_filename) fclose(dump_file);
+	return ret;
+
 }
 
 static int grib_tool_with_orderby(grib_runtime_options* options) {
@@ -161,8 +202,8 @@ static int grib_tool_with_orderby(grib_runtime_options* options) {
 		options->error=err;
 
 		if (!h) {
-			fprintf(dump_file,"\t\t*** unreadable message ***\n");
-			if (options->fail || err==GRIB_WRONG_LENGTH) GRIB_CHECK(err,0);
+			fprintf(dump_file,"\t\t*** unable to read message ***\n");
+			if (options->fail || err==GRIB_WRONG_LENGTH) GRIB_CHECK_NOLINE(err,0);
 
 			failed=(grib_failed*)grib_context_malloc_clear(c,sizeof(grib_failed));
 			failed->count=infile->handle_count;
@@ -238,7 +279,7 @@ static int grib_tool_without_orderby(grib_runtime_options* options) {
 		infile->handle_count=0;
 		infile->filter_handle_count=0;
 
-		grib_tool_new_file_action(options);
+		grib_tool_new_file_action(options,infile);
 
 
 		while((h = grib_handle_new_from_file(c,infile->file,&err))
@@ -249,7 +290,7 @@ static int grib_tool_without_orderby(grib_runtime_options* options) {
 
 			if (!h) {
 				fprintf(dump_file,"\t\t*** unreadable message ***\n");
-				if (options->fail ) GRIB_CHECK(err,0);
+				if (options->fail ) GRIB_CHECK_NOLINE(err,0);
 
 				failed=(grib_failed*)grib_context_malloc_clear(c,sizeof(grib_failed));
 				failed->count=infile->handle_count;
@@ -305,6 +346,188 @@ static int grib_tool_without_orderby(grib_runtime_options* options) {
 	return 0;
 }
 
+static int navigate(grib_field_tree* fields,grib_runtime_options* options) {
+	int err=0;
+	if (!fields || options->stop) return 0;
+
+	if (fields->field) {
+		grib_handle* h=grib_index_get_handle(fields->field,&err);
+		if (!options->index2->current) 
+			options->index2->current=grib_context_malloc_clear(options->context,sizeof(grib_field_list));
+		options->index2->current->field=fields->field;
+		if (!h) return err;
+		grib_skip_check(options,h);
+		if (options->skip && options->strict) {
+			grib_tool_skip_handle(options,h);
+	    } else {
+			grib_tool_new_handle_action(options,h);
+			grib_handle_delete(h);
+		}
+	}
+
+	err=navigate(fields->next_level,options);
+	if (err) return err;
+
+	err=navigate(fields->next,options);
+
+	return err;
+}
+
+
+static int grib_tool_index(grib_runtime_options* options) {
+	int err=0;
+	grib_context* c=NULL;
+	char* f1=options->infile->name;
+	char* f2=options->infile_extra->name;
+	grib_index_key *k1,*k2;
+	int found=0;
+
+	c=grib_context_get_default();
+
+	options->index1=grib_index_read(c,f1,&err);
+	if (err) 
+		grib_context_log(c,(GRIB_LOG_FATAL) | (GRIB_LOG_PERROR) ,
+			"unable to read index from %s",f1);
+	
+	options->index2=grib_index_read(c,f2,&err);
+	if (err) 
+		grib_context_log(c,(GRIB_LOG_FATAL) | (GRIB_LOG_PERROR) ,
+			"unable to read index from %s",f2);
+
+	k1=options->index1->keys;
+	while ( k1 ) {
+		k2=options->index2->keys;
+		found=0;
+		while (k2) {
+			if ( !strcmp(k1->name,k2->name) ) {
+				found=1;
+				break;
+			}
+			k2=k2->next;
+		}
+		if (!found) {
+			printf("Indexes contained in the input files have different keys\n");
+			printf("keys in file %s:\n",f1);
+			k1=options->index1->keys;
+			while (k1) {
+				printf("\t%s\n",k1->name);
+				k1=k1->next;
+			}
+			printf("keys in file %s:\n",f2);
+			k2=options->index2->keys;
+			while (k2) {
+				printf("\t%s\n",k2->name);
+				k2=k2->next;
+			}
+
+			exit(1);
+
+		}
+
+		k1->value[0]=0;
+		k1=k1->next;
+	}
+
+	k2=options->index2->keys;
+	while ( k2 ) {
+		k1=options->index1->keys;
+		found=0;
+		while (k1) {
+			if ( !strcmp(k1->name,k2->name) ) {
+				found=1;
+				break;
+			}
+			k1=k1->next;
+		}
+		if (!found) {
+			printf("Indexes contained in the input files have different keys\n");
+			printf("keys in file %s:\n",f2);
+			k2=options->index2->keys;
+			while (k2) {
+				printf("\t%s\n",k2->name);
+				k2=k2->next;
+			}
+			printf("keys in file %s:\n",f1);
+			k1=options->index1->keys;
+			while (k1) {
+				printf("\t%s\n",k1->name);
+				k1=k1->next;
+			}
+
+			exit(1);
+
+		}
+
+		k2=k2->next;
+	}
+
+	navigate(options->index2->fields,options);
+
+	grib_tool_finalise_action(options);
+
+	return 0;
+}
+
+static int scan(grib_context* c,grib_runtime_options* options,const char* dir) {
+	struct dirent *s;
+	DIR *d;
+	int err=0;
+
+	d= opendir(dir);
+	if (!d) {
+		grib_context_log(c,(GRIB_LOG_ERROR) | (GRIB_LOG_PERROR) , "opendir %s",dir);
+		return GRIB_IO_PROBLEM;
+	}
+
+	while ((s=readdir(d)) && (err==0)) {
+		if(strcmp(s->d_name,".") != 0 && strcmp(s->d_name,"..") != 0) {
+			char buf[1024];
+			sprintf(buf,"%s/%s",dir,s->d_name);
+			process(c,options,buf);
+		}
+	}
+	return 0;
+}
+
+
+static int process(grib_context* c,grib_runtime_options* options,const char* path) {
+	struct stat s;
+	int err=0;
+	int ioerr=0;
+	if ( (err = lstat(path,&s)) ) {
+		ioerr=errno;
+		grib_context_log(c,(GRIB_LOG_ERROR) | (GRIB_LOG_PERROR),"Cannot stat %s",path);
+		return GRIB_IO_PROBLEM;
+	}
+
+	if (S_ISDIR(s.st_mode) && !S_ISLNK(s.st_mode)) {
+		scan(c,options,path);
+	} else {
+		grib_tool_new_filename_action(options,path);
+	}
+	return 0;
+
+}
+
+static int grib_tool_onlyfiles(grib_runtime_options* options) {
+	grib_context* c=NULL;
+	grib_tools_file* infile=NULL;
+
+	c=grib_context_get_default();
+	infile=options->infile;
+
+	while (infile!=NULL && infile->name!=NULL) {
+
+		process(c,options,infile->name);
+
+		infile=infile->next;
+	}
+
+	grib_tool_finalise_action(options);
+
+	return 0;
+}
+
 static void grib_print_header(grib_runtime_options* options,grib_handle* h) {
 
 	size_t strlenkey=0;
@@ -414,6 +637,10 @@ static int to_skip(grib_handle* h,grib_values* v,int *err) {
       lvalue=grib_is_missing( h,v->name,err);
       ret = (lvalue == v->equal) ? 0 : 1;
       break;
+	default:
+	  fprintf(dump_file,"invalid type for %s\n",v->name);
+	  exit(1);
+
   }
 
   return ret;
@@ -471,7 +698,7 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h) {
 	double dvalue=0;
 	long lvalue=0;
 	char value[MAX_STRING_LEN];
-	char* notfound="not found";
+	char* notfound="not_found";
 
 	if (!options->verbose) return;
 
@@ -502,7 +729,7 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h) {
 			}
         }
 		if (ret != GRIB_SUCCESS) {
-			if (options->fail) GRIB_CHECK(ret,options->print_keys[i].name);
+			if (options->fail) GRIB_CHECK_NOLINE(ret,options->print_keys[i].name);
 			if (ret == GRIB_NOT_FOUND) strcpy(value,notfound);
 			else {
 				fprintf(dump_file,"%s %s\n",grib_get_error_message(ret),options->print_keys[i].name);
@@ -525,18 +752,7 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h) {
 	}
 
 	if (options->latlon) {
-		double min;
-		if (options->latlon_idx<0) {
-			min=options->distances[0];
-			options->latlon_idx=0;
-			i=0;
-			for (i=1;i<4;i++) {
-				if (min>options->distances[i]) {
-					min=options->distances[i];
-					options->latlon_idx=i;
-				}
-			}
-		}
+
 		if (options->latlon_mode==4){
 			int i=0;
             for (i=0;i<4;i++) {
@@ -554,14 +770,31 @@ void grib_print_key_values(grib_runtime_options* options,grib_handle* h) {
 	}
 	if (options->index_on) {
 		double v=0;
-		if (grib_get_double_element(h,"values",options->index,&v) == GRIB_SUCCESS) {
-			sprintf(value,options->format,v);
-			strlenvalue = strlen(value);
-			width = strlenvalue < options->default_print_width ?
-                    options->default_print_width + 2 :
-					strlenvalue + 2;
-			fprintf(dump_file,"%-*s",(int)width,value);
+		/*if (grib_get_double_element(h,"values",options->index,&v) != GRIB_SUCCESS) {*/
+		if (1) {
+			size_t size;
+			double* values;
+			int err=0;
+
+			err=grib_get_size(h,"values",&size);
+			if (err) {
+				sprintf(value,"unknown");
+				if (!options->fail) exit(err);
+				return;
+			}
+			values=grib_context_malloc_clear(h->context,size*sizeof(double));
+			grib_get_double_array(h,"values",values,&size);
+			v=values[options->index];
+			grib_context_free(h->context,values);		
 		}
+		
+		sprintf(value,options->format,v);
+		strlenvalue = strlen(value);
+		width = strlenvalue < options->default_print_width ?
+                    options->default_print_width + 2 :
+				strlenvalue + 2;
+		fprintf(dump_file,"%-*s",(int)width,value);
+		
 	}
 	fprintf(dump_file,"\n");
 
@@ -660,7 +893,7 @@ void grib_tools_write_message(grib_runtime_options* options, grib_handle* h) {
 			exit(1);
 		}
 	}
-	GRIB_CHECK(grib_get_message(h,&buffer,&size),0);
+	GRIB_CHECK_NOLINE(grib_get_message(h,&buffer,&size),0);
 	if (options->gts && h->gts_header)
 		fwrite(h->gts_header,1,h->gts_header_len,options->outfile->file);
 
